##############################################################################
#
# Copyright (c) 2017 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os
import sqlite3
import time

from slapos.test.promise import data
from slapos.monitor.collect import ResourceCollect

class TestMonitorCollect(unittest.TestCase):

  def setUp(self):
    self.base_path = "/".join(data.__file__.split("/")[:-1])
    self.status = "ok"

    # populate db
    self.conn = sqlite3.connect('/tmp/collector.db')
    f = open(self.base_path+"/monitor_collect.sql")
    sql = f.read()
    self.conn.executescript(sql)
    self.conn.close() 

    # inititalise
    self.collector = ResourceCollect('/tmp/')
    

  def test_getPartitionUsedMemoryAverage(self):
    self.assertEqual(1195.492578125,
      self.collector.getPartitionUsedMemoryAverage('slapuser15', '2017-09-16'))

  def test_getPartitionCPULoadAverage(self):
    self.assertEqual(2.1599999999999993,
      self.collector.getPartitionCPULoadAverage('slapuser15', '2017-09-16'))

  def test_getPartitionDiskUsedAverage(self):
    self.assertEqual(35.5234375,
      self.collector.getPartitionDiskUsedAverage('slapuser15', '2017-04-18'))

  def test_getPartitionConsumption(self):
    data = self.collector.getPartitionConsumption('slapuser15', date_scope='2017-09-16', 
                                             min_time='00:01:00', max_time='00:13:00')
    self.assertEqual(1302.66, data[0]['cpu_time'])
    self.assertEqual(26825304064.0, data[0]['io_rw_counter'])

  def test_getPartitionComsumptionStatus(self):
    data = self.collector.getPartitionComsumptionStatus('slapuser15', date_scope='2017-09-16',
                                         min_time='00:01:00', max_time='00:13:00')
    self.assertEqual(7.3, data[0]['cpu_percent'])
    self.assertEqual(2822535483392.0, data[2]['io_rw_counter'])

  def tearDown(self):
    os.remove("/tmp/collector.db") 
if __name__ == '__main__':
  unittest.main()




##############################################################################
#
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.test.promise.plugin import TestPromisePluginMixin
from slapos.grid.promise import PromiseError
import os
import sqlite3
from slapos.test.promise import data
from slapos.grid.promise import PromiseError

class TestCheckFreeDiskSpace(TestPromisePluginMixin):

  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    log_folder = os.path.join(self.partition_dir, 'var/log')
    os.makedirs(log_folder)

    self.db_file = '/tmp/collector.db'
    self.base_path = "/".join(data.__file__.split("/")[:-1])

    # populate db
    self.conn = sqlite3.connect(self.db_file)
    f = open(self.base_path+"/disktest.sql")
    sql = f.read()
    self.conn.executescript(sql)
    self.conn.close()

    self.promise_name = "check-free-disk-space.py"
    self.th_file = os.path.join(self.partition_dir, 'min-disk-value')
    with open(self.th_file, 'w') as f:
      f.write('2048')

    content = """from slapos.promise.plugin.check_free_disk_space import RunPromise

extra_config_dict = {
  'collectordb': '%(collectordb)s',
  'threshold-file': '%(th_file)s',
  'test-check-date': '2017-10-02',
}
""" % {'collectordb': self.db_file, 'th_file': self.th_file}
    self.writePromise(self.promise_name, content)

  def tearDown(self):
    TestPromisePluginMixin.tearDown(self)
    if os.path.exists(self.db_file):
      os.remove(self.db_file)

  def test_check_free_disk_with_unavailable_dates(self):
    content = """from slapos.promise.plugin.check_free_disk_space import RunPromise

extra_config_dict = {
  'collectordb': '%(collectordb)s',
  'threshold-file': '%(th_file)s',
  'test-check-date': '2017-09-14',
  'test-check-time': '18:00'
}
""" % {'collectordb': self.db_file, 'th_file': self.th_file}
    self.writePromise(self.promise_name, content)

    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(result['result']['message'], "No result from collector database: disk check skipped")

  def test_disk_space_ok(self):
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(result['result']['message'], "Disk usage: OK")

  def test_disk_space_nok(self):
    with open(self.th_file, 'w') as f:
      f.write('298927494144')
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(result['result']['message'], "Free disk space low: remaining 269.1 G (threshold: 291921381.0 G)")

  def test_check_free_disk_with_unicode_string_path(self):
    # set path unicode
    self.partition_dir = u'%s' % self.partition_dir
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(result['result']['message'], "Disk usage: OK")

if __name__ == '__main__':
  unittest.main()



from __future__ import print_function

import argparse
import os
import subprocess
import re

from .runner_utils import *

def parseArgumentList():
  parser = argparse.ArgumentParser()
  parser.add_argument('--exclude-path', action='append', required=True)

  return parser.parse_args()

def calculateSignature():
  args = parseArgumentList()

  file_list = sys.stdin.read()
  file_list = file_list.split('\0')

  if not len(file_list):
    return 0

  result = re.match('^\.\/runner\/instance\/slappart(\d)', file_list[0])
  base_relative_path = result.group()
  excluded_path_list = tuple(['./' + os.path.relpath(os.path.join(base_relative_path, path)) for path in args.exclude_path])

  filtered_file_list = [
    x for x in file_list
    if not x.startswith(excluded_path_list)
  ]

  print('\n'.join(getSha256Sum(filtered_file_list)))


#!/usr/bin/env python

import sys
import os
import glob
import json
import ConfigParser
import time
from datetime import datetime
import base64
import hashlib
import PyRSS2Gen

def getKey(item):
  return item.source.name

class MonitorFeed(object):

  def __init__(self, instance_name, hosting_name,
      public_url, private_url, feed_url):
    self.rss_item_list = []
    self.report_date = datetime.utcnow()
    self.instance_name = instance_name
    self.hosting_name = hosting_name
    self.public_url = public_url
    self.private_url = private_url
    self.feed_url = feed_url

  def appendItem(self, item_dict, has_string=""):
    event_date = item_dict['result']['change-date']
    report_date = item_dict['result']['date']
    description = item_dict['result'].get('message', '')
    rss_item = PyRSS2Gen.RSSItem(
      categories = [item_dict['status']],
      source = PyRSS2Gen.Source(item_dict['title'], self.public_url),
      title = '[%s] %s' % (item_dict['status'], item_dict['title']),
      description = "\n%s" % (description,),
      link = self.private_url,
      pubDate = event_date,
      guid = PyRSS2Gen.Guid(base64.b64encode("%s, %s, %s, %s" % (self.hosting_name,
                              item_dict['title'], has_string, event_date)),
                            isPermaLink=False)
    )
    self.rss_item_list.append(rss_item)

  def generateRSS(self, output_file):
    ### Build the rss feed
    # try to keep the list in the same order
    sorted(self.rss_item_list, key=getKey)
    rss_feed = PyRSS2Gen.RSS2 (
      title = self.instance_name,
      link = self.feed_url,
      description = self.hosting_name,
      lastBuildDate = self.report_date,
      items = self.rss_item_list
    )

    with open(output_file, 'w') as frss:
      frss.write(rss_feed.to_xml())

def generateStatisticsData(stat_file_path, content):
  # csv document for success/error statictics
  if not os.path.exists(stat_file_path) or os.stat(stat_file_path).st_size == 0:
    with open(stat_file_path, 'w') as fstat:
      data_dict = {
        "date": time.time(),
        "data": ["Date, Success, Error, Warning"]
      }
      fstat.write(json.dumps(data_dict))

  current_state = ''
  if content.has_key('state'):
    current_state = '%s, %s, %s, %s' % (
      content['date'],
      content['state']['success'],
      content['state']['error'],
      '')

  # append to file
  if current_state:
    with open (stat_file_path, mode="r+") as fstat:
      fstat.seek(0,2)
      position = fstat.tell() -2
      fstat.seek(position)
      fstat.write('%s}' % ',"{}"]'.format(current_state))

def writeDocumentList(folder_path):
  # Save document list in a file called _document_list
  public_document_list = [os.path.splitext(file)[0]
                for file in os.listdir(folder_path) if file.endswith('.json')]

  with open(os.path.join(folder_path, '_document_list'), 'w') as lfile:
    lfile.write('\n'.join(public_document_list))

def generateMonitoringData(config, public_folder, private_folder, public_url,
    private_url, feed_url):
  feed_output = os.path.join(public_folder, 'feed')
  # search for all status files
  file_list = filter(
    os.path.isfile,
    glob.glob("%s/promise/*.status.json" % public_folder)
  )

  promises_status_file = os.path.join(private_folder, '_promise_status')
  previous_state_dict = {}
  new_state_dict = {}
  error = success = 0
  monitor_feed = MonitorFeed(
    config.get('monitor', 'title'),
    config.get('monitor', 'root-title'),
    public_url,
    private_url,
    feed_url)

  if os.path.exists(promises_status_file):
    with open(promises_status_file) as f:
      try:
        previous_state_dict = json.loads(f.read())
      except ValueError:
        pass

  for file in file_list:
    try:
      with open(file, 'r') as temp_file:
        tmp_json = json.loads(temp_file.read())

      if tmp_json['result']['failed']:
        promise_status = "ERROR"
        error += 1
      else:
        promise_status = "OK"
        success += 1
      tmp_json['result']['change-date'] = tmp_json['result']['date']
      if previous_state_dict.has_key(tmp_json['name']):
        status, change_date, _ = previous_state_dict[tmp_json['name']]
        if promise_status == status:
          tmp_json['result']['change-date'] = change_date

      tmp_json['status'] = promise_status
      message_hash = hashlib.md5(tmp_json['result'].get('message', '')).hexdigest()
      new_state_dict[tmp_json['name']] = [
        promise_status,
        tmp_json['result']['change-date'],
        message_hash
      ]
      monitor_feed.appendItem(tmp_json, message_hash)
      savePromiseHistory(
        tmp_json['title'],
        tmp_json,
        previous_state_dict.get(tmp_json['name']),
        public_folder
      )
    except ValueError, e:
      # bad json file
      print "ERROR: Bad json file at: %s\n%s" % (file, str(e))
      continue

  with open(promises_status_file, "w") as f:
    json.dump(new_state_dict, f)

  monitor_feed.generateRSS(feed_output)
  return error, success

def savePromiseHistory(promise_name, state_dict, previous_state_list,
    history_folder):
  if not os.path.exists(history_folder) and os.path.isdir(history_folder):
    self.logger.warning('Bad promise history folder, history is not saved...')
    return

  history_file = os.path.join(
    history_folder,
    '%s.history.json' % promise_name
  )

  # Remove useless informations
  result = state_dict.pop('result')
  state_dict.update(result)
  state_dict.pop('path', '')
  state_dict.pop('type', '')
  if not os.path.exists(history_file) or not os.stat(history_file).st_size:
    with open(history_file, 'w') as f:
      data_dict = {
        "date": time.time(),
        "data": [state_dict]
      }
      json.dump(data_dict, f)
  else:
    if previous_state_list is not None:
      _, change_date, checksum = previous_state_list
      current_sum = hashlib.md5(state_dict.get('message', '')).hexdigest()
      if state_dict['change-date'] == change_date and \
          current_sum == checksum:
        # Only save the changes and not the same info
        return

    state_dict.pop('title', '')
    state_dict.pop('name', '')
    with open (history_file, mode="r+") as f:
      f.seek(0,2)
      f.seek(f.tell() -2)
      f.write('%s}' % ',{}]'.format(json.dumps(state_dict)))

def run(monitor_conf_file):

  config = ConfigParser.ConfigParser()
  config.read(monitor_conf_file)

  base_folder = config.get('monitor', 'private-folder')
  status_folder = config.get('monitor', 'public-folder')
  base_url = config.get('monitor', 'base-url')
  related_monitor_list = config.get("monitor", "monitor-url-list").split()
  statistic_folder = os.path.join(base_folder, 'documents')
  # need webdav to update parameters
  parameter_file = os.path.join(base_folder, 'config', '.jio_documents', 'config.json')

  public_url = "%s/share/public/" % base_url
  private_url = "%s/share/private/" % base_url
  feed_url = "%s/public/feed" % base_url
  status = 'OK'
  global_state_file = os.path.join(base_folder, 'monitor.global.json')
  public_state_file = os.path.join(status_folder, 'monitor.global.json')
  report_date = datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S')

  error, success = generateMonitoringData(config, status_folder, base_folder,
                                          public_url, private_url, feed_url)
  if error:
    status = 'ERROR'

  global_state_dict = dict(
    status=status,
    state={
      'error': error,
      'success': success
    },
    type='global', # bwd compatibility
    portal_type='Software Instance',
    date=report_date,
    _links={"rss_url": {"href": feed_url},
            "public_url": {"href": public_url},
            "private_url": {"href": private_url},
            "related_monitor": []
          },
    data={'state': 'monitor_state.data',
          'process_state': 'monitor_process_resource.status',
          'process_resource': 'monitor_resource_process.data',
          'memory_resource': 'monitor_resource_memory.data',
          'io_resource': 'monitor_resource_io.data',
          'monitor_process_state': 'monitor_resource.status'},
    title=config.get('monitor', 'title'),
    specialise_title=config.get('monitor', 'root-title'),
    aggregate_reference=config.get('promises', 'computer-id'),
    ipv4=config.get('promises', 'ipv4'),
    ipv6=config.get('promises', 'ipv6'),
    software_release=config.get('promises', 'software-release'),
    software_type=config.get('promises', 'software-type'),
    partition_id=config.get('promises', 'partition-id'),
  )

  if not global_state_dict['title']:
    global_state_dict['title'] = 'Instance Monitoring'

  if related_monitor_list:
    global_state_dict['_links']['related_monitor'] = [{'href': "%s/share/public" % url}
                          for url in related_monitor_list]

  if os.path.exists(parameter_file):
    with open(parameter_file) as cfile:
      global_state_dict['parameters'] = json.loads(cfile.read())

  # Public information with the link to private folder
  public_state_dict = dict(
    status=status,
    date=report_date,
    _links={'monitor': {'href': '%s/share/private/' % base_url}},
    title=global_state_dict.get('title', ''),
    specialise_title=global_state_dict.get('specialise_title', ''),
  )
  public_state_dict['_links']['related_monitor'] = global_state_dict['_links'].get('related_monitor', [])

  with open(global_state_file, 'w') as fglobal:
    fglobal.write(json.dumps(global_state_dict))

  with open(public_state_file, 'w') as fpglobal:
    fpglobal.write(json.dumps(public_state_dict))

  # write list of files
  writeDocumentList(status_folder)
  writeDocumentList(base_folder)
  writeDocumentList(statistic_folder)

  generateStatisticsData(
    os.path.join(statistic_folder, 'monitor_state.data.json'),
    global_state_dict
  )

  return 0

def main():
  if len(sys.argv) < 2:
    print("Usage: %s <monitor_conf_path>" % sys.argv[0])
    sys.exit(2)
  sys.exit(run(sys.argv[1]))



##############################################################################
#
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os
import sqlite3
from datetime import datetime

from slapos.test.promise import data
from slapos.promise.check_user_memory import getMemoryInfo, checkMemoryUsage

no_result_message = "No result found in collector.db."

class TestUserMemory(unittest.TestCase):

  def setUp(self):
    self.base_path = "/".join(data.__file__.split("/")[:-1])
    self.status = "ok"
    self.db_file = '/tmp/collector.db'
    self.db_dir = os.path.dirname(self.db_file)

    # populate db
    self.conn = sqlite3.connect(self.db_file)
    f = open(self.base_path+"/memtest.sql")
    sql = f.read()
    self.conn.executescript(sql)
    self.conn.close()

  def test_check_memory(self):
    self.assertEqual(
      ({
        'byte': 29544162598,
        'percent': 87.65432099725093,
      }, ""),
      getMemoryInfo(
        self.db_dir,
        datetime(2017, 9, 15, 0, 2, 0),
        'slaptestuser1',
      ),
    )
    self.assertEqual(
      (True, "User memory usage: 29544162598B (87.7%)", ""),
      checkMemoryUsage(
        self.db_dir,
        datetime(2017, 9, 15, 0, 2, 0),
        'slaptestuser1',
        threshold=29600000000,
      ),
    )
    self.assertEqual(
      (True, "User memory usage: 87.7% (29544162598B)", ""),
      checkMemoryUsage(
        self.db_dir,
        datetime(2017, 9, 15, 0, 2, 0),
        'slaptestuser1',
        threshold=88,
        unit="percent",
      ),
    )
    self.assertEqual(
      (False, "High user memory usage: 29544162598B (87.7%)", ""),
      checkMemoryUsage(
        self.db_dir,
        datetime(2017, 9, 15, 0, 2, 0),
        'slaptestuser1',
        threshold=29500000000,
      ),
    )
    self.assertEqual(
      (False, "High user memory usage: 87.7% (29544162598B)", ""),
      checkMemoryUsage(
        self.db_dir,
        datetime(2017, 9, 15, 0, 2, 0),
        'slaptestuser1',
        threshold=87,
        unit="percent",
      ),
    )

  def test_check_memory_with_unavailable_dates(self):
    self.assertEqual(
      (False, "error", no_result_message),
      checkMemoryUsage(self.db_dir, datetime(2017, 9, 14, 18, 0 , 0), "slaptestuser1", 1.0),
    )

  def tearDown(self):
    if os.path.exists(self.db_file):
      os.remove(self.db_file)
if __name__ == '__main__':
  unittest.main()




# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010-2014 Vifib SARL and Contributors.
# All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import sqlite3
import os
import pwd
import time
import json
import argparse
import psutil
from time import strftime
from datetime import datetime, timedelta

from slapos.collect.db import Database
from slapos.collect.reporter import ConsumptionReportBase

def parseArguments():
  """
  Parse arguments for monitor collector instance.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--output_folder',
                      help='Path of the folder where output files should be written.')
  parser.add_argument('--pid_file',
                      help='Path where should be written the pid of process.')
  parser.add_argument('--partition_id',
                      help='ID of the computer partition to collect data from.')
  parser.add_argument('--collector_db',
                      help='The path of slapos collect database.')

  return parser.parse_args()

# XXX The code on the class below should be dropped and prefer to use
# the slapos.collect.db.Database directly:
#  - https://lab.nexedi.com/nexedi/slapos.core/blob/master/slapos/collect/db.py
# the code duplication here is huge so be carefull to not reimplemnt what is 
# already implement.

class ResourceCollect:

  def __init__(self, db_path = None):
    # XXX this code is duplicated with slapos.collect.db.Database.__init__
    assert os.path.exists(db_path)
    if db_path.endswith("collector.db"):
      db_path = db_path[:-len("collector.db")]
    # If the database is locked, wait until 15 seconds
    # Do not try to created or update tables, access will be refused
    self.db = Database(db_path, create=False, timeout=15)
    self.consumption_utils = ConsumptionReportBase(self.db)

  def has_table(self, name):
    self.db.connect()
    check_result_cursor = self.db.select(
      table="sqlite_master",
      columns='name',
      where="type='table' AND name='%s'" % name)
    table_exists_result = zip(*check_result_cursor)
    if not len(table_exists_result) or table_exists_result[0][0] is None:
      return False
    return True

  def getPartitionCPULoadAverage(self, partition_id, date_scope):
    return self.consumption_utils.getPartitionCPULoadAverage(partition_id, date_scope)

  def getPartitionUsedMemoryAverage(self, partition_id, date_scope):
    return self.consumption_utils.getPartitionUsedMemoryAverage(partition_id, date_scope)/(1024*1024.0)

  def getPartitionDiskUsedAverage(self, partition_id, date_scope):
    return self.consumption_utils.getPartitionDiskUsedAverage(partition_id, date_scope)/1024.0

  def getPartitionConsumption(self, partition_id, where="", date_scope=None, min_time=None, max_time=None):
    """
      Query collector db to get consumed resource for last minute
    """
    self.db.connect()
    comsumption_list = []
    if where != "":
      where = "and %s" % where
    if not date_scope:
      date_scope = datetime.now().strftime('%Y-%m-%d')
    if not min_time:
      min_time = (datetime.now() - timedelta(minutes=1)).strftime('%H:%M:00')
    if not max_time:
      max_time = (datetime.now() - timedelta(minutes=1)).strftime('%H:%M:59')

    columns = """count(pid), SUM(cpu_percent) as cpu_result, SUM(cpu_time),
                MAX(cpu_num_threads), SUM(memory_percent), SUM(memory_rss), pid, SUM(io_rw_counter),
                SUM(io_cycles_counter)"""
    query_result = self.db.select("user", date_scope, columns,
                   where="partition = '%s'  and (time between '%s' and '%s') %s" % 
                   (partition_id, min_time, max_time, where),
                   group="pid", order="cpu_result desc")
    for result in query_result:
      count = int(result[0])
      if not count > 0:
        continue
      resource_dict = {
        'pid': result[6],
        'cpu_percent': round(result[1]/count, 2),
        'cpu_time': round((result[2] or 0)/(60.0), 2),
        'cpu_num_threads': round(result[3]/count, 2),
        'memory_percent': round(result[4]/count, 2),
        'memory_rss': round((result[5] or 0)/(1024*1024.0), 2),
        'io_rw_counter': round(result[7]/count, 2),
        'io_cycles_counter': round(result[8]/count, 2)
      }
      try:
        pprocess = psutil.Process(int(result[6]))
      except psutil.NoSuchProcess:
        pass
      else:
        resource_dict['name'] = pprocess.name()
        resource_dict['command'] = pprocess.cmdline()
        resource_dict['user'] = pprocess.username()
        resource_dict['date'] = datetime.fromtimestamp(pprocess.create_time()).strftime("%Y-%m-%d %H:%M:%S")
      comsumption_list.append(resource_dict)
    self.db.close()
    return comsumption_list
  
  def getPartitionComsumptionStatus(self, partition_id, where="", date_scope=None, min_time=None, max_time=None):
    self.db.connect()
    if where != "":
      where = " and %s" % where
    if not date_scope:
      date_scope = datetime.now().strftime('%Y-%m-%d')
    if not min_time:
      min_time = (datetime.now() - timedelta(minutes=1)).strftime('%H:%M:00')
    if not max_time:
      max_time = (datetime.now() - timedelta(minutes=1)).strftime('%H:%M:59') 

    colums = """count(pid), SUM(cpu_percent), SUM(cpu_time), SUM(cpu_num_threads), SUM(memory_percent), 
                SUM(memory_rss), SUM(io_rw_counter), SUM(io_cycles_counter)"""  
    query_result = self.db.select('user', date_scope, colums, 
                                  where="partition='%s' and (time between '%s' and '%s') %s" % 
                                  (partition_id, min_time, max_time, where))
    result_list = zip(*query_result)

    process_dict = memory_dict = io_dict = {}
    if len(result_list):
      result = result_list
      process_dict = {'total_process': result[0][0],
        'cpu_percent': round((result[1][0] or 0), 2),
        'cpu_time': round((result[2][0] or 0)/(60.0), 2),
        'cpu_num_threads': round((result[3][0] or 0), 2),
        'date': '%s %s' % (date_scope, min_time)
      }
      memory_dict = {'memory_percent': round((result[4][0] or 0), 2),
        'memory_rss': round((result[5][0] or 0)/(1024*1024.0), 2),
        'date': '%s %s' % (date_scope, min_time)
      }
      io_dict = {'io_rw_counter': round((result[6][0] or 0), 2),
        'io_cycles_counter': round((result[7][0] or 0), 2),
        'disk_used': 0,
        'date': '%s %s' % (date_scope, min_time)
      }
      if self.has_table('folder'):
        disk_result_cursor = self.db.select(
          "folder", date_scope,
          columns="SUM(disk_used)", 
          where="partition='%s' and (time between '%s' and '%s') %s" % (
            partition_id, min_time, max_time, where
          )
        )
    
        disk_used_sum = zip(*disk_result_cursor)
        if len(disk_used_sum) and disk_used_sum[0][0] is not None:
          io_dict['disk_used'] = round(disk_used_sum[0][0]/1024.0, 2)
    self.db.close()
    return (process_dict, memory_dict, io_dict)

def appendToJsonFile(file_path, content, stepback=2):
  with open (file_path, mode="r+") as jfile:
    jfile.seek(0, 2)
    position = jfile.tell() - stepback
    jfile.seek(position)
    jfile.write('%s}' % ',"{}"]'.format(content))

def initProcessDataFile(process_file):
  with open(process_file, 'w') as fprocess:
    data_dict = {
      "date": time.time(),
      "data": ["date, total process, CPU percent, CPU time, CPU threads"]
    }
    fprocess.write(json.dumps(data_dict))

def initMemoryDataFile(mem_file):
  with open(mem_file, 'w') as fmem:
    data_dict = {
      "date": time.time(),
      "data": ["date, memory used percent, memory used"]
    }
    fmem.write(json.dumps(data_dict))

def initIODataFile(io_file):
  with open(io_file, 'w') as fio:
    data_dict = {
      "date": time.time(),
      "data": ["date, io rw counter, io cycles counter, disk used"]
    }
    fio.write(json.dumps(data_dict))

def main():
  parser = parseArguments()
  if not os.path.exists(parser.output_folder) and os.path.isdir(parser.output_folder):
    raise Exception("Invalid ouput folder: %s" % parser.output_folder)

  if parser.pid_file:
    # Check that this process is not running
    if os.path.exists(parser.pid_file):
      with open(parser.pid_file, "r") as pidfile:
        try:
          pid = int(pidfile.read(6))
        except ValueError:
          pid = None
        if pid and os.path.exists("/proc/" + str(pid)):
          print("A process is already running with pid " + str(pid))
          exit(1)
    with open(parser.pid_file, "w") as pidfile:
      pidfile.write('%s' % os.getpid())

  # Consumption global status
  process_file = os.path.join(parser.output_folder, 'monitor_resource_process.data.json')
  mem_file = os.path.join(parser.output_folder, 'monitor_resource_memory.data.json')
  io_file = os.path.join(parser.output_folder, 'monitor_resource_io.data.json')
  resource_file = os.path.join(parser.output_folder, 'monitor_process_resource.status.json')
  status_file = os.path.join(parser.output_folder, 'monitor_resource.status.json')

  if not os.path.exists(parser.collector_db):
    print "Collector database not found..."
    initProcessDataFile(process_file)
    initMemoryDataFile(mem_file)
    initIODataFile(io_file)
    with open(status_file, "w") as status_file:
      status_file.write(json.dumps({
        "cpu_time": 0,
        "cpu_percent": 0,
        "memory_rss": 0,
        "memory_percent": 0,
        "io_rw_counter": 0,
        "date": "",
        "total_process": 0,
        "disk_used": 0,
        "io_cycles_counter": 0,
        "cpu_num_threads": 0
      }))
    with open(resource_file, "w") as resource_file:
      resource_file.write('[]')
    exit(1)

  collector = ResourceCollect(parser.collector_db)

  date_scope = datetime.now().strftime('%Y-%m-%d')
  stat_info = os.stat(parser.output_folder)
  partition_user = pwd.getpwuid(stat_info.st_uid)[0]

  process_result, memory_result, io_result = collector.getPartitionComsumptionStatus(partition_user)

  label_list = ['date', 'total_process', 'cpu_percent', 'cpu_time', 'cpu_num_threads',
                  'memory_percent', 'memory_rss', 'io_rw_counter', 'io_cycles_counter',
                  'disk_used']
  resource_status_dict = {}
  if not os.path.exists(process_file) or os.stat(process_file).st_size == 0:
    initProcessDataFile(process_file)

  if not os.path.exists(mem_file) or os.stat(mem_file).st_size == 0:
    initMemoryDataFile(mem_file)

  if not os.path.exists(io_file) or os.stat(io_file).st_size == 0:
    initIODataFile(io_file)

  if process_result and process_result['total_process'] != 0.0:
    appendToJsonFile(process_file, ", ".join(
      [str(process_result[key]) for key in label_list if process_result.has_key(key)])
    )
    resource_status_dict.update(process_result)
  if memory_result and memory_result['memory_rss'] != 0.0:
    appendToJsonFile(mem_file, ", ".join(
      [str(memory_result[key]) for key in label_list if memory_result.has_key(key)])
    )
    resource_status_dict.update(memory_result)
  if io_result and io_result['io_rw_counter'] != 0.0:
    appendToJsonFile(io_file, ", ".join(
      [str(io_result[key]) for key in label_list if io_result.has_key(key)])
    )
    resource_status_dict.update(io_result)

  with open(status_file, 'w') as fp:
    fp.write(json.dumps(resource_status_dict))

  # Consumption Resource
  resource_process_status_list = collector.getPartitionConsumption(partition_user)
  if resource_process_status_list:
    with open(resource_file, 'w') as rf:
      rf.write(json.dumps(resource_process_status_list))

  if os.path.exists(parser.pid_file):
    os.unlink(parser.pid_file)



# -*- coding: utf-8 -*-
import os, time
import sys
import shutil
import tempfile
import unittest
import json
from datetime import datetime
from slapos.monitor.runpromise import MonitorPromiseLauncher, getArgumentParser
from slapos.grid.promise.generic import PROMISE_LOG_FOLDER_NAME, PROMISE_RESULT_FOLDER_NAME

class MonitorPromiseTest(unittest.TestCase):

  def setUp(self):
    self.base_dir = tempfile.mkdtemp()
    self.etc_dir = os.path.join(self.base_dir, 'etc')
    self.output_dir = os.path.join(self.base_dir, PROMISE_RESULT_FOLDER_NAME)
    self.log_output_dir = os.path.join(self.base_dir, PROMISE_LOG_FOLDER_NAME)
    self.promise_dir = os.path.join(self.etc_dir, 'plugin')
    self.old_promise_dir = os.path.join(self.etc_dir, 'promise')
    self.public_dir = os.path.join(self.base_dir, 'public')
    self.private_dir = os.path.join(self.base_dir, 'private')
    self.run_dir = os.path.join(self.base_dir, 'run')
    self.log_dir = os.path.join(self.base_dir, 'log')
    os.mkdir(self.etc_dir)
    os.mkdir(self.promise_dir)
    os.mkdir(self.public_dir)
    os.mkdir(self.private_dir)
    os.mkdir(self.run_dir)
    os.mkdir(self.old_promise_dir)
    os.mkdir(self.log_dir)

    self.monitor_config = """[monitor]
private-folder = %(base_dir)s/private
public-folder = %(base_dir)s/public
root-title = %(root_title)s
log-folder = %(base_dir)s/log
base-url = %(base_url)s
title = %(title)s

[promises]
pid-path = %(base_dir)s/run/promises.pid
partition-folder = %(base_dir)s
computer-id = COMP-1234
partition-cert = 
partition-key = 
partition-id = slappart0
ipv6 = 2001:34c:1254:df3:89::5df3
software-release = http://some.url.com/software.cfg
master-url = http://10.0.151.118:50000
software-type = default
ipv4 = 10.0.151.118
""" % {'base_dir': self.base_dir, 'title': 'Monitor', 'root_title': 'Monitor ROOT',
       'base_url': 'https://monitor.test.com', }

    self.monitor_config_file = os.path.join(self.base_dir, 'monitor.conf')
    with open(self.monitor_config_file, 'w') as f:
      f.write(self.monitor_config)

  def tearDown(self):
    if os.path.exists(self.base_dir):
      shutil.rmtree(self.base_dir)

  def writePromiseOK(self, name):
    content = """#!/bin/sh

echo "success"
exit 0
"""
    promise_path = os.path.join(self.old_promise_dir, name)
    self.writeContent(promise_path, content)
    os.chmod(promise_path, 0755)
    return promise_path

  def writePromiseNOK(self, name):
    content = """#!/bin/sh

echo "failed"
exit 2
"""
    promise_path = os.path.join(self.old_promise_dir, name)
    self.writeContent(promise_path, content)
    os.chmod(promise_path, 0755)
    return promise_path

  def generatePromiseScript(self, name, success=True, failure_count=1, content="",
      periodicity=0.03):
    promise_content = """from zope import interface as zope_interface
from slapos.grid.promise import interface
from slapos.grid.promise import GenericPromise

class RunPromise(GenericPromise):

 zope_interface.implements(interface.IPromise)

 def __init__(self, config):
   GenericPromise.__init__(self, config)
   self.setPeriodicity(minute=%(periodicity)s)

 def sense(self):
   %(content)s

   if not %(success)s:
     self.logger.error("failed")
   else:
     self.logger.info("success")

 def anomaly(self):
   return self._anomaly(result_count=2, failure_amount=%(failure_amount)s)

 def test(self):
   return self._test(result_count=1, failure_amount=%(failure_amount)s)
""" % {'success': success, 'content': content, 'failure_amount': failure_count,
      'periodicity': periodicity}
 
    with open(os.path.join(self.promise_dir, name), 'w') as f:
      f.write(promise_content)
    return os.path.join(self.promise_dir, name)

  def writeContent(self, file_path, config):
    with open(file_path, 'w') as cfg:
      cfg.write(config)

  def getPromiseParser(self, use_config=True, check_anomaly=False, force=False):

    arg_parser = getArgumentParser()
    base_list = ['-c', self.monitor_config_file]
    if use_config:
      if check_anomaly:
        base_list.append('-a')
      if force:
        base_list.append('--force')
      return arg_parser.parse_args(base_list)

    pid_path = os.path.join(self.run_dir, 'monitor-promise.pid')
    promise_cmd = [
      '--pid-path',
      '%s' % pid_path, '-o', self.public_dir,
      '--master-url', 'http://10.0.151.118:50000',
      '--partition-id', 'slappart0',
      '--computer-id', 'COMP-1234']
    return arg_parser.parse_args(promise_cmd)

  def test_promise_generic(self):
    promise_file = self.generatePromiseScript('my_promise.py', success=True)
    promise_file2 = self.generatePromiseScript('my_second_promise.py', success=True)
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'my_promise.status.json')
    os.system('cat %s' % result_file)
    self.assertTrue(os.path.exists(result_file))
    my_result = json.loads(open(result_file).read().decode("utf-8"))
    my_result['result'].pop('date')
    expected_result = {
      u'title': u'my_promise', u'name': u'my_promise.py',
      u'result': {
        u'failed': False, u'message': u'success', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s/my_promise.py' % self.promise_dir,
    }
    self.assertEqual(expected_result, my_result)

    result_file = os.path.join(self.output_dir, 'my_second_promise.status.json')
    self.assertTrue(os.path.exists(result_file))
    second_result = json.loads(open(result_file).read().decode("utf-8"))
    second_result['result'].pop('date')

    expected_result = {
      u'title': u'my_second_promise', u'name': u'my_second_promise.py',
      u'result': {
        u'failed': False, u'message': u'success', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s/my_second_promise.py' % self.promise_dir,
    }
    self.assertEqual(expected_result, second_result)

  def test_promise_generic_failed(self):
    promise_file = self.generatePromiseScript('my_promise.py', success=False)
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'my_promise.status.json')
    self.assertTrue(os.path.exists(result_file))
    my_result = json.loads(open(result_file).read().decode("utf-8"))
    my_result['result'].pop('date')

    expected_result = {
      u'title': u'my_promise', u'name': u'my_promise.py',
      u'result': {
        u'failed': True, u'message': u'failed', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s/my_promise.py' % self.promise_dir,
    }
    self.assertEqual(expected_result, my_result)

  def test_promise_generic_status_change(self):
    promise_file = self.generatePromiseScript('my_promise.py', success=False)
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'my_promise.status.json')
    self.assertTrue(os.path.exists(result_file))
    my_result = json.loads(open(result_file).read().decode("utf-8"))
    my_result['result'].pop('date')

    expected_result = {
      u'title': u'my_promise', u'name': u'my_promise.py',
      u'result': {
        u'failed': True, u'message': u'failed', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s/my_promise.py' % self.promise_dir,
    }
    self.assertEqual(expected_result, my_result)

    os.system('rm %s/*.pyc' % self.promise_dir)
    time.sleep(2)
    self.generatePromiseScript('my_promise.py', success=True)
    promise_runner2 = MonitorPromiseLauncher(parser)
    promise_runner2.start()

    my_result = json.loads(open(result_file).read().decode("utf-8"))
    my_result['result'].pop('date')

    expected_result = {
      u'title': u'my_promise', u'name': u'my_promise.py',
      u'result': {
        u'failed': False, u'message': u'success', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s/my_promise.py' % self.promise_dir,
    }
    self.assertEqual(expected_result, my_result)

  def test_promise_generic_periodicity(self):
    promise_file = self.generatePromiseScript('my_promise.py', success=False, periodicity=0.03)
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'my_promise.status.json')
    self.assertTrue(os.path.exists(result_file))

    # run again
    os.unlink(result_file)
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()
    self.assertFalse(os.path.exists(result_file))

    time.sleep(2)
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()
    self.assertTrue(os.path.exists(result_file))

  def test_promise_generic_run_only(self):
    promise_file = self.generatePromiseScript('my_promise.py', success=False)
    second_promise = self.generatePromiseScript('my_second_promise.py', success=False)
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'my_promise.status.json')
    second_result = os.path.join(self.output_dir, 'my_second_promise.status.json')
    self.assertTrue(os.path.exists(result_file))
    self.assertTrue(os.path.exists(second_result))

    config = getArgumentParser().parse_args(['-c', self.monitor_config_file, '-a', '-f',
                                      '--run-only', 'my_second_promise.py'])
    os.unlink(result_file)
    promise_runner = MonitorPromiseLauncher(config)
    promise_runner.start()
    self.assertFalse(os.path.exists(result_file))
    self.assertTrue(os.path.exists(second_result))

  def test_promise_OK(self):
    promise = self.writePromiseOK('promise_1')
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'promise_1.status.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read().decode("utf-8"))
    start_date = datetime.strptime(result1['result'].pop('date'), '%Y-%m-%dT%H:%M:%S')

    expected_result = {
      u'title': u'promise_1', u'name': u'promise_1',
      u'result': {
        u'failed': False, u'message': u'success', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s' % promise,
    }

    # second run
    parser = self.getPromiseParser(force=True)
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()
    result2 = json.loads(open(result_file).read().decode("utf-8"))
    start_date2 = datetime.strptime(result2['result'].pop('date'), '%Y-%m-%dT%H:%M:%S')
    self.assertEqual(expected_result, result2)

  def test_promise_two_folder(self):

    promise = self.writePromiseOK('promise_1')
    promise2 = self.writePromiseOK('promise_2')
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'promise_1.status.json')
    result2_file = os.path.join(self.output_dir, 'promise_2.status.json')
    self.assertTrue(os.path.exists(result_file))
    self.assertTrue(os.path.exists(result2_file))
    result1 = json.loads(open(result_file).read().decode("utf-8"))
    start_date = datetime.strptime(result1['result'].pop('date'), '%Y-%m-%dT%H:%M:%S')

    expected_result = {
      u'title': u'promise_1', u'name': u'promise_1',
      u'result': {
        u'failed': False, u'message': u'success', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s' % promise,
    }
    self.assertEqual(expected_result, result1)

    result2 = json.loads(open(result2_file).read())
    start_date2 = datetime.strptime(result2['result'].pop('date'), '%Y-%m-%dT%H:%M:%S')

    expected_result = {
      u'title': u'promise_2', u'name': u'promise_2',
      u'result': {
        u'failed': False, u'message': u'success', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s' % promise2,
    }
    self.assertEqual(expected_result, result2)

  def test_promise_NOK(self):
    promise = self.writePromiseNOK('promise_1')
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'promise_1.status.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read().decode("utf-8"))
    result1['result'].pop('date')
    expected_result = {
      u'title': u'promise_1', u'name': u'promise_1',
      u'result': {
        u'failed': True, u'message': u'failed', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s' % promise,
    }
    self.assertEqual(expected_result, result1)

    # second run
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()
    result2 = json.loads(open(result_file).read().decode("utf-8"))
    result2['result'].pop('date')
    self.assertEqual(expected_result, result2)

  def test_promise_mixed(self):
    promise = self.writePromiseOK('promise_1')
    parser = self.getPromiseParser()
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result_file = os.path.join(self.output_dir, 'promise_1.status.json')
    self.assertTrue(os.path.exists(result_file))
    result1 = json.loads(open(result_file).read().decode("utf-8"))
    result1['result'].pop('date')
    expected_result = {
      u'title': u'promise_1', u'name': u'promise_1',
      u'result': {
        u'failed': False, u'message': u'success', u'type': u'Test Result'
        },
      u'execution-time': 0.1,
      u'path': u'%s' % promise,
    }
    self.assertEqual(expected_result, result1)

    # second run with failure
    time.sleep(1)
    promise = self.writePromiseNOK('promise_1')
    parser = self.getPromiseParser(force=True)
    expected_result['result']['message'] = u'failed'
    expected_result['result']['failed'] = True
    promise_runner = MonitorPromiseLauncher(parser)
    promise_runner.start()

    result2 = json.loads(open(result_file).read().decode("utf-8"))
    result2['result'].pop('date')
    self.assertEqual(expected_result, result2)




from libcloud.types import Provider
from libcloud.providers import get_driver
import paramiko
import StringIO

driver_list = {
    'DUMMY': get_driver(Provider.DUMMY),
    'EC2_US_EAST': get_driver(Provider.EC2_US_EAST),
    'EC2_US_WEST': get_driver(Provider.EC2_US_WEST),
    'EC2_EU_WEST': get_driver(Provider.EC2_EU_WEST),
    'RACKSPACE': get_driver(Provider.RACKSPACE),
    'SLICEHOST': get_driver(Provider.SLICEHOST),
    'GOGRID': get_driver(Provider.GOGRID),
    'VPSNET': get_driver(Provider.VPSNET),
    'LINODE': get_driver(Provider.LINODE),
    'VCLOUD': get_driver(Provider.VCLOUD),
    'RIMUHOSTING': get_driver(Provider.RIMUHOSTING),
    'ECP': get_driver(Provider.ECP),
    'IBM': get_driver(Provider.IBM),
    'OPENNEBULA': get_driver(Provider.OPENNEBULA),
    'DREAMHOST': get_driver(Provider.DREAMHOST),
  }
def getDriverInstance(driverName, key, secret=None, secure=True, host=None,
    port=None):
  return driver_list.get(driverName)(key, secret, secure, host, port)

def getSSHConnection(driver, hostname, private_key):
  client = paramiko.SSHClient()
  client.load_system_host_keys()
  client.set_missing_host_key_policy(paramiko.WarningPolicy())
  #TODO if exception : try DSSKey
  pkey = paramiko.RSAKey.from_private_key(StringIO.StringIO(private_key))
  client.connect(hostname=hostname, username='root', pkey=pkey,
        look_for_keys=False)
  return client
  
def getNode(driver, node_uuid):
  node_list = [node for node in driver.list_nodes() if node_uuid in node.uuid]
  if len(node_list) == 0:
    return None
  if len(node_list) != 1:
    raise IndexError('Several nodes with the uuid %r exist.' % node_uuid)
  return node_list[0]



##############################################################################
#
# Copyright (c) 2017 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os.path
import socket
import time
from slapos.promise.check_error_on_apache_log import test

from slapos.test.promise import data

class TestCheckErrorOnApacheLog(unittest.TestCase):

  def get_time(self, sec):
    return time.strftime("%a %b %d %H:%M:%S %Y", time.localtime(time.time()-sec))

  def _update_logs(self):
    log_file_list = [
       "apache_error_log",
       "infoonly_error_log",
       "timeout_error_log",
       "unreachable_error_log"]
    i = 600
    for log_file in log_file_list:
      new = ""
      old = ""
      with open(self.base_path + "/" + log_file) as f:
        for line in f:
          new += line.replace("DATETIME", self.get_time(i))
          old += line.replace("DATETIME", self.get_time(i+3600))
          i -= 1

      with open(self.base_path + "/SOFTINST-0_" + log_file, "w") as f:
        f.write(old)
        f.write(new)

  def setUp(self):
    self.base_path = "/".join(data.__file__.split("/")[:-1])
    self._update_logs()

  def test_no_error(self):
    self.assertEqual("OK",
      test(self.base_path + "/SOFTINST-0_infoonly_error_log", 0))

    self.assertEqual("OK",
      test(self.base_path + "/SOFTINST-0_infoonly_error_log", 3600))

  def test_error(self):
    self.assertEqual("ERROR=2 (NOTROUTE=2, UNREACHEABLENET=0, TIMEOUT=0)",
      test(self.base_path + "/SOFTINST-0_apache_error_log", 0))

    self.assertEqual("ERROR=1 (NOTROUTE=1, UNREACHEABLENET=0, TIMEOUT=0)",
      test(self.base_path + "/SOFTINST-0_apache_error_log", 3600))

  def test_error_timeout(self):
    self.assertEqual("ERROR=4 (NOTROUTE=0, UNREACHEABLENET=0, TIMEOUT=4)",
      test(self.base_path + "/SOFTINST-0_timeout_error_log", 0))

    self.assertEqual("ERROR=2 (NOTROUTE=0, UNREACHEABLENET=0, TIMEOUT=2)",
      test(self.base_path + "/SOFTINST-0_timeout_error_log", 3600))

  def test_error_unreacheabler(self):
    self.assertEqual("ERROR=11 (NOTROUTE=0, UNREACHEABLENET=11, TIMEOUT=0)",
      test(self.base_path + "/SOFTINST-0_unreachable_error_log", 0))

    self.assertEqual("ERROR=11 (NOTROUTE=0, UNREACHEABLENET=11, TIMEOUT=0)",
      test(self.base_path + "/SOFTINST-0_unreachable_error_log", 3600))   
  
if __name__ == '__main__':
  unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os
import stat
import json
import ConfigParser
import traceback
import argparse
import urllib2
import ssl
import glob
import socket
from datetime import datetime

OPML_START = """<?xml version="1.0" encoding="UTF-8"?>
<!-- OPML generated by SlapOS -->
<opml version="1.1">
	<head>
		<title>%(root_title)s</title>
		<dateCreated>%(creation_date)s</dateCreated>
		<dateModified>%(modification_date)s</dateModified>
	</head>
	<body>
	  <outline text="%(outline_title)s">"""
OPML_END = """	  </outline>
  </body>
</opml>"""

OPML_OUTLINE_FEED = '<outline text="%(title)s" title="%(title)s" type="rss" version="RSS" htmlUrl="%(html_url)s" xmlUrl="%(xml_url)s" url="%(global_url)s" />'


def parseArguments():
  """
  Parse arguments for monitor instance.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('-c', '--config-file', required=True,
                      default='monitor.cfg',
                      help='Monitor Configuration file')

  return parser.parse_args()


def mkdirAll(path):
  try:
    os.makedirs(path)
  except OSError, e:
    if e.errno == os.errno.EEXIST and os.path.isdir(path):
      pass
    else: raise

def softConfigGet(config, *args, **kwargs):
  try:
    return config.get(*args, **kwargs)
  except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
    return None

def createSymlink(source, destination):
  try:
    os.symlink(source, destination)
  except OSError, e:
    if e.errno != os.errno.EEXIST:
      raise

class Monitoring(object):

  def __init__(self, configuration_file):
    self._config_file = configuration_file
    config = self.loadConfig([configuration_file])

    # Set Monitor variables
    self.title = config.get("monitor", "title")
    self.root_title = config.get("monitor", "root-title")
    self.service_pid_folder = config.get("monitor", "service-pid-folder")
    self.crond_folder = config.get("monitor", "crond-folder")
    self.public_folder = config.get("monitor", "public-folder")
    self.private_folder = config.get("monitor", "private-folder")
    self.webdav_folder = config.get("monitor", "webdav-folder")
    self.webdav_url = '%s/share' % config.get("monitor", "base-url")
    self.public_url = '%s/public' % config.get("monitor", "base-url")
    self.public_path_list = config.get("monitor", "public-path-list").split()
    self.private_path_list = config.get("monitor", "private-path-list").split()
    self.monitor_url_list = config.get("monitor", "monitor-url-list").split()
    self.parameter_list = [param.strip() for param in config.get("monitor", "parameter-list").split('\n') if param]
    # Use this file to write knowledge0_cfg required by webrunner
    self.parameter_cfg_file = config.get("monitor", "parameter-file-path").strip()
    self.pid_file = config.get("monitor", "pid-file")
    self.promise_output_file = config.get("monitor", "promise-output-file")
    self.promise_folder = config.get("promises", 'promise-folder')
    self.legacy_promise_folder = config.get("promises", 'legacy-promise-folder')
    self.promise_output = config.get("promises", 'output-folder')

    self.config_folder = os.path.join(self.private_folder, 'config')
    self.data_folder = config.get("monitor", "document-folder")
    self.bootstrap_is_ok = True

  def loadConfig(self, pathes, config=None):
    if config is None:
      config = ConfigParser.ConfigParser()
    try:
      config.read(pathes)
    except ConfigParser.MissingSectionHeaderError:
      traceback.print_exc()
    return config

  def readInstanceConfiguration(self):
    type_list = ['raw', 'file', 'htpasswd', 'httpdcors']
    configuration_list = []

    if not self.parameter_list:
      return []
  
    for config in self.parameter_list:
      config_list = config.strip().split(' ')
      # type: config_list[0]
      if len(config_list) >= 3 and config_list[0] in type_list:
        if config_list[0] == 'raw':
          configuration_list.append(dict(
            key='',
            title=config_list[1],
            value=' '.join(config_list[2:])
          ))
        elif (config_list[0] == 'file' or config_list[0] == 'htpasswd'):
          directory = os.path.dirname(config_list[2])
          if not os.path.exists(directory) or not os.access(directory, os.W_OK):
            raise OSError("Directory does not exists or does not have write acess")
          if os.path.exists(config_list[2]) and os.path.isfile(config_list[2]):
            try:
              with open(config_list[2]) as cfile:
                param_value = cfile.read()
            except OSError, e:
              print 'Cannot read file %s, Error is: %s' % (config_list[2], str(e))
              pass
          else:
            param_value = ""
          parameter = dict(
            key=config_list[1],
            title=config_list[1],
            value=param_value,
            description={
              "type": config_list[0],
              "file": config_list[2]
            }
          )
          if config_list[0] == 'htpasswd':
            if len(config_list) != 5 or not os.path.exists(config_list[4]):
              print 'htpasswd file is not specified: %s' % str(config_list)
              continue
            parameter['description']['user'] = config_list[3]
            parameter['description']['htpasswd'] = config_list[4]
          configuration_list.append(parameter)
        elif config_list[0] == 'httpdcors' and os.path.exists(config_list[2]) and \
            os.path.exists(config_list[3]):
          old_cors_file = os.path.join(
            os.path.dirname(config_list[2]),
            'prev_%s' % os.path.basename(config_list[2])
          )
          try:
            cors_content = ""
            if os.path.exists(old_cors_file):
              with open(old_cors_file) as cfile:
                cors_content = cfile.read()
            else:
              # Create empty file
              with open(old_cors_file, 'w') as cfile:
                cfile.write("")
            parameter = dict(
              key=config_list[1],
              title=config_list[1],
              value=cors_content,
              description={
                "type": config_list[0],
                "cors_file": config_list[2],
                "gracefull_bin": config_list[3]
              }
            )
            configuration_list.append(parameter)
          except OSError, e:
            print 'Cannot read file at %s, Error is: %s' % (old_cors_file, str(e))
            pass
    return configuration_list

  def createSymlinksFromConfig(self, destination_folder, source_path_list, name=""):
    if destination_folder:
      if source_path_list:
        for path in source_path_list:
          path = path.rstrip('/')
          dirname = os.path.join(destination_folder, name)
          try:
            mkdirAll(dirname)  # could also raise OSError
            os.symlink(path, os.path.join(dirname, os.path.basename(path)))
          except OSError, e:
            if e.errno != os.errno.EEXIST:
              raise

  def getMonitorTitleFromUrl(self, monitor_url):
    # This file should be generated
    if not monitor_url.startswith('https://') and not monitor_url.startswith('http://'):
      return 'Unknown Instance'
    if not monitor_url.endswith('/'):
      monitor_url = monitor_url + '/'

    url  = monitor_url + '/monitor.global.json'
    success = False
    monitor_title = 'Unknown Instance'
    try:
      # Timeout after 20 seconds
      timeout = 20
      # XXX - working here with public url
      if hasattr(ssl, '_create_unverified_context'):
        context = ssl._create_unverified_context()
        response = urllib2.urlopen(url, context=context, timeout=timeout)
      else:
        response = urllib2.urlopen(url, timeout=timeout)
    except urllib2.HTTPError:
      print "ERROR: Failed to get Monitor configuration file at %s " % url
    except socket.timeout, e:
      print "ERROR: Timeout while downloading monitor config at %s " % url
    else:
      try:
        monitor_dict = json.loads(response.read())
        monitor_title = monitor_dict.get('title', 'Unknown Instance')
        success = True
      except ValueError, e:
        print "ERROR: Json file at %s is not valid" % url

    self.bootstrap_is_ok = success
    return monitor_title

  def configureFolders(self):
    # create symlinks from monitor.conf
    self.createSymlinksFromConfig(self.public_folder, self.public_path_list)
    self.createSymlinksFromConfig(self.private_folder, self.private_path_list)
    # configure public and private folder
    self.createSymlinksFromConfig(self.webdav_folder, [self.public_folder])
    self.createSymlinksFromConfig(self.webdav_folder, [self.private_folder])

    config_jio_folder = os.path.join(self.config_folder, '.jio_documents')
    mkdirAll(config_jio_folder)

  def makeConfigurationFiles(self):
    config_folder = os.path.join(self.config_folder, '.jio_documents')
    parameter_config_file = os.path.join(config_folder, 'config.parameters.json')
    parameter_file = os.path.join(config_folder, 'config.json')

    parameter_list = self.readInstanceConfiguration()
    description_dict = {}

    if parameter_list:
      for i in range(0, len(parameter_list)):
        key = parameter_list[i]['key']
        if key:
          description_dict[key] = parameter_list[i].pop('description')

    with open(parameter_config_file, 'w') as config_file:
      config_file.write(json.dumps(description_dict))

    with open(parameter_file, 'w') as config_file:
      config_file.write(json.dumps(parameter_list))

    try:
      with open(self.parameter_cfg_file, 'w') as pfile:
        pfile.write('[public]\n')
        for parameter in parameter_list:
          if parameter['key']:
            pfile.write('%s = %s\n' % (parameter['key'], parameter['value']))
    except OSError, e:
      print "Error failed to create file %s" % self.parameter_cfg_file
      pass


  def generateOpmlFile(self, feed_url_list, output_file):

    if os.path.exists(output_file):
      creation_date = datetime.utcfromtimestamp(os.path.getctime(output_file))\
        .strftime("%a, %d %b %Y %H:%M:%S +0000")
      modification_date = datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S +0000")
    else:
      creation_date = modification_date = datetime.utcnow()\
        .strftime("%a, %d %b %Y %H:%M:%S +0000")

    opml_content = OPML_START % {'creation_date': creation_date,
                                  'modification_date': modification_date,
                                  'outline_title': 'Monitoring RSS Feed list',
                                  'root_title': self.root_title}

    opml_content += OPML_OUTLINE_FEED % {'title': self.title,
        'html_url': self.public_url + '/feed',
        'xml_url': self.public_url + '/feed',
        'global_url': "%s/private/" % self.webdav_url}
    for feed_url in feed_url_list:
      opml_content += OPML_OUTLINE_FEED % {
        'title': self.getMonitorTitleFromUrl(feed_url + "/share/public/"),
        'html_url': feed_url + '/public/feed',
        'xml_url': feed_url + '/public/feed',
        'global_url': "%s/share/private/" % feed_url}

    opml_content += OPML_END

    with open(output_file, 'w') as wfile:
      wfile.write(opml_content)

  def cleanupMonitorDeprecated(self):
    # Monitor report feature is removed
    cleanup_file_list = glob.glob("%s/*.history.json" % self.private_folder)
    cleanup_file_list.extend(glob.glob("%s/*.report.json" % self.private_folder))
    cleanup_file_list.extend(glob.glob("%s/*.report.json" % self.data_folder))
    cleanup_file_list.extend(glob.glob("%s/*.history.json" % self.data_folder))
    cleanup_file_list.extend(glob.glob("%s/*.status.json" % self.public_folder))
    cleanup_file_list.append(self.crond_folder + "/monitor-reports")
    cleanup_file_list.append(self.crond_folder + "/monitor-promises")

    for file in cleanup_file_list:
      try:
        if os.path.exists(file):
          os.unlink(file)
      except OSError, e:
        print "failed to remove file %s." % file, str(e)

    # cleanup result of promises that was removed
    promise_list = os.listdir(self.legacy_promise_folder)
    for name in os.listdir(self.promise_folder):
      if name.startswith('__init__'):
        continue
      promise_list.append(os.path.splitext(name)[0])
    for name in os.listdir(self.promise_output):
      if not name.endswith('.status.json'):
        continue
      try:
        position = promise_list.index(name.replace('.status.json', ''))
      except ValueError:
        status_path = os.path.join(self.promise_output, name)
        if os.path.exists(status_path):
          try:
            os.unlink(status_path)
          except OSError, e:
            print "Error: Failed to delete %s" % status_path, str(e)
      else:
        promise_list.pop(position)

  def bootstrapMonitor(self):

    if os.path.exists(self.promise_output_file):
      os.unlink(self.promise_output_file)

    # save pid of current process into file
    with open(self.pid_file, 'w') as pid_file:
      pid_file.write(str(os.getpid()))

    self.configureFolders()

    # Generate OPML file
    self.generateOpmlFile(self.monitor_url_list,
      os.path.join(self.public_folder, 'feeds'))

    # cleanup deprecated entries
    self.cleanupMonitorDeprecated()

    # Generate parameters files and scripts
    self.makeConfigurationFiles()

    # Write an empty file when monitor bootstrap went until the end
    if self.bootstrap_is_ok:
      with open(self.promise_output_file, 'w') as promise_file:
        promise_file.write("")
      print "SUCCESS: bootstrap is OK"

    return 0


def main():
  parser = parseArguments()

  monitor = Monitoring(parser.config_file)
  
  sys.exit(monitor.bootstrapMonitor())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import csv
import datetime
import json
import httplib
import os
import shutil
import socket
import subprocess
import sys
import time
import traceback
import urllib2
import urlparse
import uuid

def createStatusItem(item_directory, instance_name, callback, date, link, status):
  global app
  callback_short_name = os.path.basename(callback)
  content = json.dumps({
    'title': '%s-PBS %s : %s' % (instance_name, callback_short_name, status),
    'description': '%s run at %s' % (callback_short_name, datetime.datetime.fromtimestamp(date).isoformat()),
    'pubDate': date,
    'link': link,
  })

  item_path = os.path.join(item_directory, "status_%s" % time.time())
  with open(item_path, 'w') as file:
    file.write(content)


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument('-l', '--log', nargs=1, required=True,
                      dest='logfile', metavar='logfile',
                      help="Logging file")
  parser.add_argument('-t', '--title', nargs=1, required=True,
                      help="Entry title.")
  parser.add_argument('-f', '--feed', nargs=1, required=True,
                      dest='feed_url', help="Url of the feed.")
  parser.add_argument('--notification-url', dest='notification_url',
                      nargs='*', required=True,
                      help="Notification url")
  parser.add_argument('--executable', nargs=1, dest='executable',
                      help="Executable to wrap")
  parser.add_argument('--transaction-id', nargs=1, dest='transaction_id',
                      type=int, required=False,
                      help="Additional parameter for notification-url")
  parser.add_argument('--max-run', dest='max_run',
                      type=int, default=1, required=False,
                      help="Run executable until it ends correctly, in a limit of max-run times")

  # Verbose mode
  parser.add_argument('--instance-root-name', dest='instance_root_name',
                      type=str, required=False,
                      help="Path to config file containing info on instance")
  parser.add_argument('--log-url', required=False, dest='log_url',
                      help="URL where the log file will be accessible")
  parser.add_argument('--status-item-directory', dest='status_item_directory',
                      required=False, default='', type=str,
                      help="Directory containing PBS status to publish as feed.")

  args = parser.parse_args()

  if args.instance_root_name and args.log_url and args.status_item_directory:
    # Verbose mode
    saveStatus = lambda status: createStatusItem(args.status_item_directory,
                                  args.instance_root_name,
                                  args.executable[0],
                                  time.time(),
                                  args.log_url,
                                  status)
  else:
    saveStatus = lambda status: None

  saveStatus('STARTED')

  if args.max_run <= 0:
    print "--max-run argument takes a strictely positive number as argument"
    sys.exit(-1)

  while args.max_run > 0:
    try:
      content = subprocess.check_output(
          args.executable[0],
          stderr=subprocess.STDOUT
      )
      exit_code = 0
      content = ("OK</br><p>%s ran successfully</p>"
                    "<p>Output is: </p><pre>%s</pre>" % (
            args.executable[0],
            content.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
        ))
      saveStatus('FINISHED')
      break
    except subprocess.CalledProcessError as e:
      args.max_run -= 1
      saveStatus('ERROR')
      content = e.output
      exit_code = e.returncode
      content = ("FAILURE</br><p>%s Failed with returncode <em>%d</em>.</p>"
                    "<p>Output is: </p><pre>%s</pre>" % (
            args.executable[0],
            exit_code,
            content.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
        ))

  print content

  # Write feed safely
  error_message = ""
  temp_file = args.logfile[0] + '.tmp'
  try:
    shutil.copy2(args.logfile[0], temp_file)
  except IOError:
    # previous feed can be rotated, only pass
    pass
  try:
    with open(temp_file, 'a') as file_:
      csvfile = csv.writer(file_)
      csvfile.writerow([
        int(time.time()),
        args.title[0],
        content,
        'slapos:%s' % uuid.uuid4(),
      ])
    os.rename(temp_file, args.logfile[0])
  except Exception, e:
    error_message = "ERROR ON WRITING FEED - %s" % str(e)
  finally:
    try:
      os.remove(temp_file)
    except OSError:
      pass

  if error_message:
    saveStatus(error_message)
    exit_code = 1

  if exit_code != 0:
    sys.exit(exit_code)

  print 'Fetching %s feed...' % args.feed_url[0]

  feed = urllib2.urlopen(args.feed_url[0])
  body = feed.read()

  some_notification_failed = False
  for notif_url in args.notification_url:
    notification_url = urlparse.urlparse(notif_url)

    notification_port = notification_url.port
    if notification_port is None:
      notification_port = socket.getservbyname(notification_url.scheme)

    notification_path = notification_url.path
    if not notification_path.endswith('/'):
      notification_path += '/'

    transaction_id = args.transaction_id[0] if args.transaction_id else int(time.time()*1e6)
    notification_path += str(transaction_id)

    headers = {'Content-Type': feed.info().getheader('Content-Type')}
    try:
      notification = httplib.HTTPConnection(notification_url.hostname,
                                            notification_port)
      notification.request('POST', notification_path, body, headers)
      response = notification.getresponse()
      if not (200 <= response.status < 300):
        error_message = ("The remote server at %s didn't send a successful reponse.\n"
                         "Its response was %r\n" % (notif_url, response.reason))
        some_notification_failed = True
    except socket.error as exc:
      error_message = "Connection with remote server at %s failed:\n" % notif_url
      error_message += traceback.format_exc(exc)
      some_notification_failed = True
    finally:
      if error_message:
        sys.stderr.write(error_message)
        saveStatus('ERROR ON NOTIFYING : %s' % error_message)

  if some_notification_failed:
    sys.exit(1)

  saveStatus('OK')

if __name__ == '__main__':
  main()




##############################################################################
#
# Copyright (c) 2015 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os.path
from slapos.agent.agent import AutoSTemp, TestMap

TESTMAP_DICT = {
  "test-wendelin-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/wendelin/software.cfg", 
    "supply_computer": "COMP-2", 
    "group": "COMP-2", 
    "title": "test-wendelin-software-release"}, 
  "test-agent-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/agent/software.cfg", 
    "supply_computer": "COMP-2", 
    "group": "COMP-2", 
    "title": "test-agent-software-release"}, 
  "test-powerdns-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/powerdns/software.cfg", 
    "supply_computer": "COMP-2", 
    "group": "COMP-2", 
    "title": "test-powerdns-software-release"}, 
  "test-monitor-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/monitor/software.cfg", 
    "supply_computer": "COMP-2", 
    "group": "COMP-2", 
    "title": "test-monitor-software-release"}, 
  "test-slapos-master-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/slapos-master/software.cfg", 
    "supply_computer": "COMP-1", 
    "group": "COMP-1", 
    "title": "test-slapos-master-software-release"}, 
  "test-webrunner-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/slaprunner/software.cfg", 
    "supply_computer": "COMP-1", 
    "group": "COMP-1", 
    "title": "test-webrunner-software-release"}, 
  "test-re6stnetmaster-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/re6stnet/software.cfg", 
    "supply_computer": "COMP-2", 
    "group": "COMP-2", 
    "title": "test-re6stnetmaster-software-release"}, 
  "test-erp5testnode-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/erp5testnode/software.cfg", 
    "supply_computer": "COMP-1", 
    "group": "COMP-1", 
    "title": "test-erp5testnode-software-release"}, 
  "test-apache-frontend-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/apache-frontend/software.cfg", 
    "supply_computer": "COMP-1", 
    "group": "COMP-1", 
    "title": "test-apache-frontend-software-release"}, 
  "test-nayuos-software-release": {
    "url": "https://lab.nexedi.com/nexedi/slapos/raw/1.0/software/nayuos/software.cfg", 
    "supply_computer": "COMP-1", 
    "group": "COMP-1", 
    "title": "test-nayuos-software-release"}
  }



class TestTestMap(unittest.TestCase):

  def setUp(self):
    self.test_map = TestMap(TESTMAP_DICT)

  def test_group(self):
    """ Check available group of tests """
    self.assertEqual(self.test_map.getGroupList(), 
          ["COMP-1", "COMP-2"])

  def test_getnextgroup(self):
    """ Get Next Group """
    self.assertEqual(self.test_map.getNextGroup([]),
      "COMP-1")
    self.assertEqual(
      self.test_map.getNextGroup(ignore_list=["COMP-1"]),
      "COMP-2")

  def test_getexcludelist(self):
    """ Check available group of tests """
    self.assertEqual(self.test_map.getExcludeList("COMP-1"),
      set(['test-agent-software-release',
           'test-wendelin-software-release',
           'test-monitor-software-release',
           'test-re6stnetmaster-software-release',
           'test-powerdns-software-release']))
    self.assertEqual(
      self.test_map.getExcludeList("COMP-2"),
      set(['test-apache-frontend-software-release',
           'test-slapos-master-software-release',
           'test-webrunner-software-release',
           'test-erp5testnode-software-release',
           'test-apache-frontend-software-release',
           'test-nayuos-software-release']))

    self.test_map.addRanTest('test-agent-software-release')
    self.assertEqual(
      self.test_map.getExcludeList("COMP-2"),
      set(['test-apache-frontend-software-release',
           'test-slapos-master-software-release',
           'test-webrunner-software-release',
           'test-erp5testnode-software-release',
           'test-apache-frontend-software-release',
           'test-nayuos-software-release',
           'test-agent-software-release']))

  def test_dropgroup(self):
    """ Check available group of tests """
    test_map_copy = TestMap(TESTMAP_DICT)
    self.assertEqual(test_map_copy.getGroupList(),
          ["COMP-1", "COMP-2"])

    test_map_copy.dropGroup("COMP-1")
    self.assertEqual(test_map_copy.getGroupList(),
          ["COMP-2"])

  def test_cleanup_empty_group(self):
    """ Check available group of tests """
    test_map_copy = TestMap(TESTMAP_DICT)
    test_map_copy.test_map_dict["NEWGROUP"] = {}
    self.assertEqual(test_map_copy.getGroupList(),
          ["COMP-1", "COMP-2", "NEWGROUP"])

    test_map_copy.cleanEmptyGroup()
    self.assertEqual(test_map_copy.getGroupList(),
          ["COMP-1", "COMP-2"])

class TestAutoSTemp(unittest.TestCase):

  def test_autostemp(self):
    """ Test AutoSTemp creates the file with content and
        removes it when deleted.
    """
    f = AutoSTemp("foo")
    self.assertEqual(open(f.name, "r").read(), "foo")
    fname = f.name
    self.assertTrue(os.path.isfile(fname))
    del f
    self.assertFalse(os.path.isfile(fname))

if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2013 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import argparse
import json
import importlib
import logging
import os
import sys
import tempfile
import time
import traceback
from erp5.util import taskdistribution
try:
  from erp5.util.testnode import Utils
except ImportError:
  pass

def importFrom(name):
  """
  Import a test suite module (in the suites module) and return it.
  """
  return importlib.import_module('.suites.%s' % name, package=__name__)

def parseArguments():
  """
  Parse arguments.
  """
  parser = argparse.ArgumentParser()
  parser.add_argument('--test-result-path', '--test_result_path',
                      metavar='ERP5_TEST_RESULT_PATH',
                      help='ERP5 relative path of the test result')

  parser.add_argument('--revision',
                      metavar='REVISION',
                      help='Revision of the test_suite')

  parser.add_argument('--test-suite', '--test_suite',
                      metavar='TEST_SUITE',
                      help='Name of the test suite')

  parser.add_argument('--test-suite-title', '--test_suite_title',
                      metavar='TEST_SUITE',
                      help='The test suite title')

  parser.add_argument('--node-title', '--test_node_title',
                      metavar='NODE_TITLE',
                      help='Title of the testnode which is running this'
                            'launcher')

  parser.add_argument('--node_quantity', help='Number of parallel tests to run',
                      default=1, type=int)

  parser.add_argument('--test-suite-master-url', '--master_url',
                      metavar='TEST_SUITE_MASTER_URL',
                      help='Url to connect to the ERP5 Master testsuite taskditributor')

  parser.add_argument('--log-path', '--log_path',
                      metavar='LOG_PATH',
                      help='Log Path')

  parser.add_argument('additional_arguments', nargs=argparse.REMAINDER)

  return parser.parse_args()

def setupLogging(name=__name__, log_path=None):
  logger_format = '%(asctime)s %(name)-13s: %(levelname)-8s %(message)s'
  formatter = logging.Formatter(logger_format)
  logging.basicConfig(level=logging.INFO, format=logger_format)

  root_logger = logging.getLogger('')
  fd, fname = tempfile.mkstemp()
  file_handler = logging.FileHandler(fname)
  file_handler.setFormatter(formatter)
  root_logger.addHandler(file_handler)

  if log_path:
    file_handler = logging.handlers.RotatingFileHandler(
        filename=log_path,
        maxBytes=20000000, backupCount=4)
    file_handler.setFormatter(formatter)
    root_logger.addHandler(file_handler)

  logger = logging.getLogger(name)
  return logger, fname

def runTestSuite(test_suite_title, test_suite_arguments, logger):
  """
  Run a specified test suite, by dynamically loading the module and calling
  its "runTestSuite" method.
  """
  try:
    # Generate the additional arguments that were given using the syntax
    # additionalargument1=value1 additionalargument2=value2
    parsed_arguments = dict(key.split('=') for key in test_suite_arguments)
    test_suite_module = importFrom(test_suite_title)
    success = test_suite_module.runTestSuite(**parsed_arguments)
  except:
    logger.exception('Impossible to run resiliency test:')
    success = False
  return success

class ScalabilityTest(object):
  """
  Simple structure carrying test data.
  """
  def __init__(self, data, test_result):
    self.__dict__ = {}
    self.__dict__.update(data)
    self.test_result = test_result

class ScalabilityLauncher(object):
  """
  Core part of the code, responsible of speaking with the ERP5 testnode Master
  and running tests.
  """
  def __init__(self):
    self._argumentNamespace = parseArguments()
    if self._argumentNamespace.log_path:
      log_path = os.path.join(self._argumentNamespace.log_path,
                              'runScalabilityTestSuite.log')
    else:
      log_path = None
    logger, fname = setupLogging('runScalabilityTestSuite', log_path)
    self.log = logger.info

    # Proxy to erp5 master test_result
    self.test_result = taskdistribution.TestResultProxy(
                        self._argumentNamespace.test_suite_master_url,
                        1.0, logger,
                        self._argumentNamespace.test_result_path,
                        self._argumentNamespace.node_title,
                        self._argumentNamespace.revision
                      )

  def getNextTest(self):
    """
    Return a ScalabilityTest with current running test case informations,
    or None if no test_case ready
    """
    data = self.test_result.getRunningTestCase()
    if data == None:
      return None
    decoded_data = Utils.deunicodeData(json.loads(
                  data
                ))
    next_test = ScalabilityTest(decoded_data, self.test_result)
    return next_test

  def run(self):
    self.log('Resiliency Launcher started, with:')
    self.log('Test suite master url: %s' % self._argumentNamespace.test_suite_master_url)
    self.log('Test suite: %s' % self._argumentNamespace.test_suite)
    self.log('Test result path: %s' % self._argumentNamespace.test_result_path)
    self.log('Revision: %s' % self._argumentNamespace.revision)
    self.log('Node title: %s' % self._argumentNamespace.node_title)

    while True:
      time.sleep(5)
      current_test = self.getNextTest()
      if current_test == None:
        self.log('No Test Case Ready')
      else:
        start_time = time.time()
        error_message_set, exit_status = set(), 0

        proxy = taskdistribution.ServerProxy(
                    self._argumentNamespace.test_suite_master_url,
                    allow_none=True
                ).portal_task_distribution
        retry_time = 2.0
        test_result_line_test = taskdistribution.TestResultLineProxy(
                                  proxy, retry_time, self.log,
                                  current_test.relative_path,
                                  current_test.title
                                )

        success = runTestSuite(
            self._argumentNamespace.test_suite,
            self._argumentNamespace.additional_arguments,
            self.log,
        )

        if success:
          error_count = 0
        else:
          error_count = 1

        test_duration = time.time() - start_time
        test_result_line_test.stop(stdout='Success',
                        test_count=1,
                        error_count=error_count,
                        duration=test_duration)
        self.log('Test Case Stopped')

    return error_message_set, exit_status

def runResiliencyTest():
  """
  Used for automated test suite run from "Scalability" Test Node infrastructure.
  It means the instance running this code should have been deployed by a
  "Scalability" Testnode.
  """
  error_message_set, exit_status = ScalabilityLauncher().run()
  for error_message in error_message_set:
    print >>sys.stderr, 'ERROR: %s' % error_message

  sys.exit(exit_status)

def runUnitTest():
  """
  Function meant to be run by "classical" (a.k.a UnitTest) erp5testnode.
  """
  args = parseArguments()
  logger, fname = setupLogging('runScalabilityTestSuite', None)
  try:
    master = taskdistribution.TaskDistributor(args.test_suite_master_url)
    test_suite_title = args.test_suite_title or args.test_suite
    revision = args.revision

    test_result = master.createTestResult(revision, [test_suite_title],
      args.node_title, True, test_suite_title, 'foo')

    if test_result is None:
      # No test to run.
      logger.info("There is no test to run. Exiting...")
      return

    test_line = test_result.start()
    start_time = time.time()

    args.additional_arguments.append('type=UnitTest')
    success = runTestSuite(
        args.test_suite,
        args.additional_arguments,
        logger,
    )

    if success:
      error_count = 0
    else:
      error_count = 1

    test_duration = time.time() - start_time
    max_size = 4096
    fsize = os.stat(fname).st_size
    with open(fname) as f:
      if fsize <= max_size:
        stdout = f.read()
      else:
        # Read only 2048 bytes from the file. The whole
        # file will be available on the server.
        stdout = f.read(2048)
        stdout += "\n[...] File truncated\n"
        f.seek(-2048, os.SEEK_END)
        stdout += f.read()

    test_line.stop(stdout=stdout,
                    test_count=1,
                    error_count=error_count,
                    duration=test_duration)
  except:
    raise
  finally:
    os.remove(fname)



from zope import interface as zope_interface
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise
import os
import time
import psutil
from .util import tail_file

class RunPromise(GenericPromise):

  zope_interface.implements(interface.IPromise)

  def __init__(self, config):
    GenericPromise.__init__(self, config)
    self.setPeriodicity(minute=2)

  def sense(self):
    process_pid_file = self.getConfig('process-pid-file')
    if not os.path.exists(process_pid_file):
      self.logger.info("Bootstrap didn't run!")
      return

    with open(process_pid_file) as f:
      try:
        pid = int(f.read())
      except ValueError, e:
        raise ValueError("%r is empty or doesn't contain a valid pid number: %s" % (
          process_pid_file, str(e)))

    try:
      process = psutil.Process(pid)
      command_string = ' '.join(process.cmdline())
      if "monitor.bootstrap" in command_string and \
          self.getPartitionFolder() in command_string:
        for i in range(0, 15):
          if process.is_running():
            time.sleep(1)
          else:
            break
        else:
          self.logger.error("Monitor bootstrap is running for more than 15 seconds!")
          return
    except psutil.NoSuchProcess:
      # process exited
      pass

    status_file = self.getConfig('status-file')
    if os.path.exists(status_file) and not os.stat(status_file).st_size:
      self.logger.info("Bootstrap OK")
      return

    message = "Monitor bootstrap exited with error."
    log_file = os.path.join(self.getPartitionFolder(), ".%s_%s.log" % (
      self.getConfig('partition-id'),
      self.getConfig('process-name')))
    if os.path.exists(log_file):
      message += "\n ---- Latest monitor-boostrap.log ----\n"
      message += tail_file(log_file, 4)

    self.logger.error(message)

  def test(self):
    return self._test(result_count=1, failure_amount=1)

  def anomaly(self):
    # bang if we have 3 error successively
    return self._anomaly(result_count=3, failure_amount=3)



# -*- coding: utf-8 -*-
import os, time
import sys
import shutil
import tempfile
import unittest
import json

from slapos.monitor.monitor import Monitoring

class MonitorBootstrapTest(unittest.TestCase):

  def setUp(self):
    self.base_dir = tempfile.mkdtemp()
    os.mkdir(os.path.join(self.base_dir, 'plugin'))
    os.mkdir(os.path.join(self.base_dir, 'promise'))
    os.mkdir(os.path.join(self.base_dir, 'public'))
    os.mkdir(os.path.join(self.base_dir, 'private'))
    os.mkdir(os.path.join(self.base_dir, 'cron.d'))
    os.mkdir(os.path.join(self.base_dir, 'log'))
    os.mkdir(os.path.join(self.base_dir, 'log-backup'))
    os.mkdir(os.path.join(self.base_dir, 'logrotate.d'))
    os.mkdir(os.path.join(self.base_dir, 'monitor-report'))
    os.mkdir(os.path.join(self.base_dir, 'webdav'))
    os.mkdir(os.path.join(self.base_dir, 'run'))
    os.mkdir(os.path.join(self.base_dir, 'private/documents'))
    self.writeContent(os.path.join(self.base_dir, 'param'), '12345')
    self.writeContent(os.path.join(self.base_dir, '.monitor_pwd'), 'bcuandjy')
    self.writeContent(os.path.join(self.base_dir, 'test-httpd-cors.cfg'), '')
    self.writeContent(os.path.join(self.base_dir, 'monitor-htpasswd'), '12345')
    self.monitor_config_file = os.path.join(self.base_dir, 'monitor.conf')

    self.monitor_config_dict = dict(
      monitor_conf=self.monitor_config_file,
      base_dir=self.base_dir,
      root_title="Monitor ROOT",
      title="Monitor",
      url_list="",
      base_url="https://monitor.test.com",
      promise_runner_pid=os.path.join(self.base_dir, 'run', 'monitor-promises.pid'),
      public_folder=os.path.join(self.base_dir, 'public'),
      public_path_list="",
      private_path_list="",
      promise_run_script="/bin/echo",
      collect_run_script="/bin/echo",
      statistic_script="/bin/echo"
    )
    self.monitor_conf = """[monitor]
parameter-file-path = %(base_dir)s/knowledge0.cfg
service-pid-folder = %(base_dir)s/run
private-folder = %(base_dir)s/private
public-folder = %(base_dir)s/public
public-path-list = %(public_path_list)s
private-path-list = %(private_path_list)s
crond-folder = %(base_dir)s/cron.d
logrotate-folder = %(base_dir)s/logrotate.d
root-title = %(root_title)s
pid-file =  %(base_dir)s/monitor.pid
log-folder = %(base_dir)s/log
log-backup-folder = %(base_dir)s/log-backup
document-folder = %(base_dir)s/private/documents
parameter-list = 
  raw monitor-user admin
  file sample %(base_dir)s/param
  htpasswd monitor-password %(base_dir)s/.monitor_pwd admin %(base_dir)s/monitor-htpasswd
  httpdcors cors-domain %(base_dir)s/test-httpd-cors.cfg /bin/echo

webdav-folder = %(base_dir)s/webdav
collect-script = %(collect_run_script)s
statistic-script = %(statistic_script)s
python = python
monitor-url-list = %(url_list)s
collector-db = 
base-url = %(base_url)s
title = %(title)s
service-pid-folder = %(base_dir)s/run
promise-output-file = %(base_dir)s/monitor-bootstrap-status
promise-runner = %(promise_run_script)s
randomsleep = /bin/echo sleep

[promises]
output-folder = %(base_dir)s/public
legacy-promise-folder = %(base_dir)s/promise
promise-folder = %(base_dir)s/plugin
partition-folder = %(base_dir)s
"""

    self.opml_outline = """<outline text="%(title)s" title="%(title)s" type="rss" version="RSS" htmlUrl="%(base_url)s/public/feed" xmlUrl="%(base_url)s/public/feed" url="%(base_url)s/share/private/" />"""

  def tearDown(self):
    if os.path.exists(self.base_dir):
      shutil.rmtree(self.base_dir)

  def writeContent(self, file_path, config):
    with open(file_path, 'w') as cfg:
      cfg.write(config)

  def configPromiseList(self, amount):
    promise_dir = os.path.join(self.base_dir, 'promise')
    plugin_dir = os.path.join(self.base_dir, 'plugin')
    promse_content = "/bin/bash echo something"
    for index in range(1, amount+1):
      promise_file = os.path.join(promise_dir, 'monitor_promise-%s' % index)
      self.writeContent(promise_file, promse_content)
      os.chmod(promise_file, 0755)
    for index in range(1, amount+1):
      promise_file = os.path.join(plugin_dir, 'monitor_promise-%s.py' % index)
      self.writeContent(promise_file, promse_content)
      os.chmod(promise_file, 0644)

  def checkOPML(self, url_list):
    opml_title = "<title>%(root_title)s</title>" % self.monitor_config_dict
    self.assertTrue(os.path.exists(os.path.join(self.base_dir, 'public/feeds')))

    with open(os.path.join(self.base_dir, 'public/feeds')) as f:
      opml_content = f.read()
      self.assertTrue(opml_title in opml_content)
      for url in url_list:
        opml_outline = self.opml_outline % dict(
          title=self.monitor_config_dict['title'],
          base_url=url)
      self.assertTrue(opml_outline in opml_content)

  def check_folder_equals(self, source, destination):
    self.assertTrue(os.path.isdir(source))

    if not destination.endswith('/'):
      destination += '/'
    dest_file_list = os.listdir(destination)
    source_file_list = os.listdir(source)
    self.assertEqual(dest_file_list, source_file_list)

  def check_symlink(self, source, destination):
    if source.endswith('/'):
      source.rstrip('/')
    if destination.endswith('/'):
      destination.rstrip('/')
    self.assertTrue(os.path.islink(destination))

    source_basename = os.path.basename(source)
    dest_basename = os.path.basename(destination)
    self.assertEqual(source_basename, dest_basename)

    if os.path.isdir(source):
      self.check_folder_equals(source, destination)

  def test_monitor_bootstrap_empty(self):
    config_content = self.monitor_conf % self.monitor_config_dict
    self.writeContent(self.monitor_config_file, config_content)

    instance = Monitoring(self.monitor_config_file)
    instance.bootstrapMonitor()
    promise_file = os.path.join(self.base_dir, 'monitor-bootstrap-status')
    self.assertTrue(os.path.exists(promise_file))

    self.checkOPML([self.monitor_config_dict['base_url']])

  def test_monitor_bootstrap_check_folder(self):

    folder_one = os.path.join(self.base_dir, 'folderOne')
    folder_two = os.path.join(self.base_dir, 'folderTwo')
    file_public = os.path.join(self.base_dir, 'file_public')
    private_one = os.path.join(self.base_dir, 'privateOne')
    private_two = os.path.join(self.base_dir, 'privateTwo')
    file_private = os.path.join(self.base_dir, 'file_private')

    os.mkdir(folder_one)
    os.mkdir(folder_two)
    os.mkdir(private_one)
    os.mkdir(private_two)

    self.writeContent(file_public, 'toto')
    self.writeContent(private_two+'/toto1', 'toto')
    self.writeContent(private_two+'/toto2', 'toto')
    self.writeContent(private_two+'/toto3', 'toto')
    self.writeContent(folder_two+'/toto1', 'toto')
    self.writeContent(folder_two+'/toto2', 'toto')
    self.writeContent(folder_two+'/toto3', 'toto')

    self.monitor_config_dict['public_path_list'] = '\n  '.join([
      folder_one,
      folder_two,
      file_public
    ])
    self.monitor_config_dict['private_path_list'] = '\n  '.join([
      private_one,
      private_two,
      file_private
    ])
    config_content = self.monitor_conf % self.monitor_config_dict
    self.writeContent(self.monitor_config_file, config_content)

    instance = Monitoring(self.monitor_config_file)
    instance.bootstrapMonitor()
    promise_file = os.path.join(self.base_dir, 'monitor-bootstrap-status')
    self.assertTrue(os.path.exists(promise_file))

    self.checkOPML([self.monitor_config_dict['base_url']])

    # check symlink configured
    self.check_symlink(folder_one, os.path.join(self.base_dir, 'public', 'folderOne'))
    self.check_symlink(folder_two, os.path.join(self.base_dir, 'public', 'folderTwo'))
    self.check_symlink(file_public, os.path.join(self.base_dir, 'public', 'file_public'))
    self.check_symlink(private_one, os.path.join(self.base_dir, 'private', 'privateOne'))
    self.check_symlink(private_two, os.path.join(self.base_dir, 'private', 'privateTwo'))
    self.check_symlink(file_private, os.path.join(self.base_dir, 'private', 'file_private'))

    # public and private folder are also accessible via webdav
    self.check_symlink(os.path.join(self.base_dir, 'public'),
      os.path.join(self.base_dir, 'webdav', 'public'))
    self.check_symlink(os.path.join(self.base_dir, 'private'),
      os.path.join(self.base_dir, 'webdav', 'private'))

    # check that configuration folder exist
    self.assertTrue(os.path.exists(os.path.join(self.base_dir, 'private/config')))
    # Check jio webdav folder
    self.assertTrue(os.path.exists(os.path.join(self.base_dir, 'private/config/.jio_documents')))
    self.assertTrue(os.path.exists(os.path.join(self.base_dir, 'private/documents')))

  def test_monitor_bootstrap_promises(self):
    self.configPromiseList(3)

    config_content = self.monitor_conf % self.monitor_config_dict
    self.writeContent(self.monitor_config_file, config_content)

    instance = Monitoring(self.monitor_config_file)
    instance.bootstrapMonitor()
    promise_file = os.path.join(self.base_dir, 'monitor-bootstrap-status')
    self.assertTrue(os.path.exists(promise_file))

    self.checkOPML([self.monitor_config_dict['base_url']])

    # Check update promises_list
    self.configPromiseList(5) # Add two promises
    os.unlink(os.path.join(self.base_dir, 'promise', 'monitor_promise-2')) # drop promise number 2
    instance2 = Monitoring(self.monitor_config_file)
    instance2.bootstrapMonitor()
    self.assertTrue(os.path.exists(promise_file))

  def test_monitor_bootstrap_genconfig(self):
    config_content = self.monitor_conf % self.monitor_config_dict
    self.writeContent(self.monitor_config_file, config_content)

    instance = Monitoring(self.monitor_config_file)
    instance.bootstrapMonitor()
    promise_file = os.path.join(self.base_dir, 'monitor-bootstrap-status')
    self.assertTrue(os.path.exists(promise_file))

    self.checkOPML([self.monitor_config_dict['base_url']])

    instance_config = os.path.join(instance.config_folder, '.jio_documents', 'config.json')
    self.assertTrue(os.path.exists(instance_config))
    config_content = json.loads(open(instance_config).read())
    self.assertEqual(len(config_content), 4)
    key_list = ['', 'sample', 'monitor-password', 'cors-domain']
    for parameter in config_content:
      if parameter['key'] in key_list:
        key_list.pop(key_list.index(parameter['key']))
      if parameter['key'] == '':
        self.assertEqual(parameter, dict(
          key="",
          title="monitor-user",
          value="admin"))
      if parameter['key'] == 'sample':
        self.assertEqual(parameter, dict(
          key="sample",
          title="sample",
          value="12345"))
      if parameter['key'] == 'monitor-password':
        self.assertEqual(parameter, dict(
          key="monitor-password",
          title="monitor-password",
          value="bcuandjy"))
      if parameter['key'] == 'cors-domain':
        self.assertEqual(parameter, dict(
          key="cors-domain",
          title="cors-domain",
          value=""))

    self.assertEqual(key_list, [])





import mock
import os
import string
import random
import supervisor
import thread
import unittest


import slapos.runner.utils as runner_utils

import sys
sys.modules['slapos.runner.utils'].sup_process = mock.MagicMock()


class TestRunnerBackEnd(unittest.TestCase):
  def setUp(self):
    self.sup_process = runner_utils.sup_process
    self.sup_process.reset_mock()
    runner_utils.open = open

  def tearDown(self):
    garbage_file_list = [
      os.path.join(*(os.getcwd(), '.htpasswd')),
      os.path.join(*(os.getcwd(), '.turn-left')),
      os.path.join(*(os.getcwd(), 'slapos-test.cfg')),
    ]
    for garbage_file in garbage_file_list:
      if os.path.exists(garbage_file):
        os.remove(garbage_file)

  def _startSupervisord(self):
    cwd = os.getcwd()
    supervisord_config_file = os.path.join(cwd, 'supervisord.conf')
    open(supervisord_config_file, 'w').write("""
    """)
    supervisord = supervisor.supervisord.Supervisord('-c', supervisord_config_file)
    thread.start_new_thread()

  def test_UserCanLoginAndUpdateCredentials(self):
    """
    * Create a user with createNewUser
    * Tests user can login with checkUserCredential
    * Updates user password updateUserCredential
    * Checks user can login with new credentials
    """
    def generate_password():
      return "".join(random.sample( \
        string.ascii_letters + string.digits + string.punctuation, 20))

    config = {'etc_dir': os.getcwd()}
    login = "admin"
    password = generate_password()
    self.assertTrue(runner_utils.createNewUser(config, login, password))
    self.assertTrue(runner_utils.checkUserCredential(config, login, password))

    new_password = generate_password()
    self.assertNotEqual(password, new_password)
    runner_utils.updateUserCredential(config, login, new_password)
    self.assertTrue(runner_utils.checkUserCredential(config, login, new_password))

  @mock.patch('os.path.exists')
  def test_getCurrentSoftwareReleaseProfile(self, mock_path_exists):
    """
    * Mock a .project file
    * Tests that getCurrentSoftwareReleaseProfile returns an absolute path
    """
    cwd = os.getcwd()

    # If .project file doesn't exist, then getCurrentSoftwareReleaseProfile
    # returns an empty string
    config = {'etc_dir': os.path.join(cwd, 'etc'),
              'workspace': os.path.join(cwd, 'srv', 'runner'),
              'software_profile': 'software.cfg'}

    profile = runner_utils.getCurrentSoftwareReleaseProfile(config)
    self.assertEqual(profile, "")

    # If .project points to a SR that doesn't exist, returns empty string
    runner_utils.open = mock.mock_open(read_data="workspace/fake/path/")
    mock_path_exists.return_value = False
    profile = runner_utils.getCurrentSoftwareReleaseProfile(config)
    self.assertEqual(profile, "")

    # If software_profile exists, getCurrentSoftwareReleaseProfile should
    # return its absolute path
    runner_utils.open = mock.mock_open(read_data = "workspace/project/software/")
    mock_path_exists.return_value = True
    profile = runner_utils.getCurrentSoftwareReleaseProfile(config)
    self.assertEqual(profile, os.path.join(config['workspace'], 'project',
        'software', config['software_profile']))

  @mock.patch('os.mkdir')
  @mock.patch('slapos.runner.utils.updateProxy')
  @mock.patch('slapos.runner.utils.requestInstance')
  @mock.patch('slapos.runner.utils.config_SR_folder')
  def _runSlapgridWithLockMakesCorrectCallsToSupervisord(self,
                                                         run_slapgrid_function,
                                                         process_name,
                                                         mock_configSRFolder,
                                                         mock_requestInstance,
                                                         mock_updateProxy,
                                                         mock_mkdir):
    """
    Tests that runSoftwareWithLock and runInstanceWithLock make correct calls
    to sup_process (= supervisord)
    """
    mock_updateProxy.return_value = True
    cwd = os.getcwd()
    config = {'etc_dir': cwd,
              'software_root': os.path.join(cwd, 'software'),
              'software_log': os.path.join(cwd, 'software.log'),
              'instance_root': os.path.join(cwd, 'software'),
              'instance_log': os.path.join(cwd, 'software.log')}
    # If process is already running, then does nothing
    self.sup_process.isRunning.return_value = True
    self.assertEqual(run_slapgrid_function(config), 1)
    self.assertFalse(self.sup_process.runProcess.called)

    # If the slapgrid process is not running, it should start it
    self.sup_process.isRunning.return_value = False
    # First, without Lock
    run_slapgrid_function(config)
    self.sup_process.runProcess.assert_called_once_with(config, process_name)
    self.assertFalse(self.sup_process.waitForProcessEnd.called)
    # Second, with Lock
    self.sup_process.reset_mock()
    run_slapgrid_function(config, lock=True)
    self.sup_process.runProcess.assert_called_once_with(config, process_name)
    self.sup_process.waitForProcessEnd.assert_called_once_with(config, process_name)

  def test_runSoftwareWithLockMakesCorrectCallstoSupervisord(self):
    self._runSlapgridWithLockMakesCorrectCallsToSupervisord(
      runner_utils.runSoftwareWithLock, 'slapgrid-sr')

  def test_runInstanceWithLockMakesCorrectCallstoSupervisord(self):
    self._runSlapgridWithLockMakesCorrectCallsToSupervisord(
      runner_utils.runInstanceWithLock, 'slapgrid-cp')

  @mock.patch('os.path.exists')
  @mock.patch('os.remove')
  @mock.patch('slapos.runner.utils.startProxy')
  @mock.patch('slapos.runner.utils.stopProxy')
  @mock.patch('slapos.runner.utils.removeProxyDb')
  def test_changingSRUpdatesProjectFileWithExistingPath(self,
                                                        mock_removeProxyDb,
                                                        mock_stopProxy,
                                                        mock_startProxy,
                                                        mock_remove,
                                                        mock_path_exists):
    cwd = os.getcwd()
    config = {'etc_dir' : os.path.join(cwd, 'etc'),
              'workspace': os.path.join(cwd, 'srv', 'runner')}
    projectpath = 'workspace/project/software/'
    self.assertNotEqual(runner_utils.realpath(config, projectpath, \
                                              check_exist=False), '')

    # If projectpath doesn't exist, .project file shouldn't be written
    mock_path_exists.return_value = False
    result = runner_utils.configNewSR(config, projectpath)
    self.assertFalse(result)

    # If projectpath exist, .project file should be overwritten
    mock_path_exists.return_value = True
    runner_utils.open = mock.mock_open()
    result = runner_utils.configNewSR(config, projectpath)
    self.assertTrue(result)
    runner_utils.open.assert_has_calls([mock.call().write(projectpath)])

  @mock.patch('slapos.runner.utils.isInstanceRunning')
  @mock.patch('slapos.runner.utils.svcStopAll')
  def test_removingInstanceStopsProcessesAndCleansInstanceDirectory(self,
                                                                    mock_svcStopAll,
                                                                    mock_isInstanceRunning):
    """
    When removing the current running instances, processes should be stopped
    and directories deleted properly
    """
    cwd = os.getcwd()
    config = {'database_uri': os.path.join(cwd, 'proxy.db'),
              'etc_dir': os.path.join(cwd, 'etc'),
              'instance_root': os.path.join(cwd, 'instance'),}

    # If slapos node is running, removeCurrentInstance returns a string
    mock_isInstanceRunning.return_value = True
    self.assertTrue(isinstance(runner_utils.removeCurrentInstance(config), str))
    self.assertTrue(mock_isInstanceRunning.called)

    # If slapos is not running, process should be stopped and directories emptied
    mock_isInstanceRunning.return_value = False
    result = runner_utils.removeCurrentInstance(config)
    self.assertTrue(mock_svcStopAll.called)
    self.sup_process.stopProcess.assert_called_with(config, 'slapproxy')

  @mock.patch('os.listdir')
  @mock.patch('os.path.exists')
  @mock.patch('slapos.runner.utils.removeCurrentInstance')
  @mock.patch('slapos.runner.utils.removeSoftwareRootDirectory')
  def test_removingUsedSoftwareReleaseCleansInstancesToo(self,
                                                         mock_removeSoftwareRootDirectory,
                                                         mock_removeCurrentInstance,
                                                         mock_path_exists,
                                                         mock_listdir):
    """
    When removing the Software Release on which depends the current running
    instances, the current instances should be stopped and removed properly.
    """
    # mock_listir is needed for not raising in loadSoftwareRList or future equivalent
    mock_listdir.return_value = []

    cwd = os.getcwd()
    config = {'etc_dir': os.path.join(cwd, 'etc'),
              'software_root': os.path.join(cwd, 'software'),
              'software_link': os.path.join(cwd, 'softwareLink'),}

    self.sup_process.isRunning.return_value = False

    # First tests that if the current instance doesn't extend the Software
    # Release to delete, the instance isn't deleted
    runner_utils.open = mock.mock_open(read_data="/workspace/software/another/")
    runner_utils.removeSoftwareByName(config, '1234567890', 'my_software_name')

    self.assertFalse(mock_removeCurrentInstance.called)
    self.assertTrue(mock_removeSoftwareRootDirectory.called)

    # If the current Instance extends the Software Release, then both must
    # be removed
    mock_removeSoftwareRootDirectory.reset_mock()

    runner_utils.open = mock.mock_open(read_data="/workspace/software/my_software_name/")
    runner_utils.removeSoftwareByName(config, '1234567890', 'my_software_name')

    self.assertTrue(mock_removeCurrentInstance.called)
    self.assertTrue(mock_removeSoftwareRootDirectory.called)

  @mock.patch('slapos.runner.utils.runInstanceWithLock')
  @mock.patch('slapos.runner.utils.runSoftwareWithLock')
  def test_runSoftwareRunOnlyOnceIfSoftwareSuccessfullyCompiledOnFirstTime(self,
                                                                           mock_runSoftwareWithLock,
                                                                           mock_runInstanceWithLock):
    cwd = os.getcwd()
    config = {'runner_workdir': cwd,
              'etc_dir': cwd}

    build_and_run_parameter_dict = {
      'run_instance': False,
      'run_software': True,
      'max_run_instance': 3,
      'max_run_software': 3,
    }
    runner_utils.saveBuildAndRunParams(config, build_and_run_parameter_dict)

    # First, configuration is set to only run the compilation of the software release
    # Both runSoftwareWithLock and runInstanceWithLock succeed on 1st try
    mock_runSoftwareWithLock.return_value = 0
    mock_runInstanceWithLock.return_value = 0

    runner_utils.runSlapgridUntilSuccess(config, 'software')
    self.assertEqual(mock_runSoftwareWithLock.call_count, 1)
    self.assertEqual(mock_runInstanceWithLock.call_count, 0)

    # Second, instanciation should start if compilation succeeded
    mock_runSoftwareWithLock.reset_mock()
    build_and_run_parameter_dict.update({'run_instance': True})
    runner_utils.saveBuildAndRunParams(config, build_and_run_parameter_dict)

    runner_utils.runSlapgridUntilSuccess(config, 'software')
    self.assertEqual(mock_runSoftwareWithLock.call_count, 1)
    self.assertEqual(mock_runInstanceWithLock.call_count, 1)

  @mock.patch('slapos.runner.utils.runInstanceWithLock')
  @mock.patch('slapos.runner.utils.runSoftwareWithLock')
  def test_runSoftwareDonotRestartForeverEvenIfBuildoutFileIsWrong(self,
                                                                   mock_runSoftwareWithLock,
                                                                   mock_runInstanceWithLock):
    """
    Restarting compilation or instanciation should happen a limited number of
    times to prevent useless runs due to a mistaken buildout config.
    """
    cwd = os.getcwd()
    config = {'runner_workdir': cwd,
              'etc_dir': cwd}

    build_and_run_parameter_dict = {
      'run_instance': True,
      'run_software': True,
      'max_run_instance': 3,
      'max_run_software': 3,
    }
    runner_utils.saveBuildAndRunParams(config, build_and_run_parameter_dict)

    # runSoftwareWithLock always fail and runInstanceWithLock succeeds on 1st try
    mock_runSoftwareWithLock.return_value = 1
    mock_runInstanceWithLock.return_value = 0

    runner_utils.runSlapgridUntilSuccess(config, 'software')
    self.assertEqual(mock_runSoftwareWithLock.call_count,
                     build_and_run_parameter_dict['max_run_software'])
    # if running software fails, then no need to try to deploy instances
    self.assertEqual(mock_runInstanceWithLock.call_count, 0)

  @mock.patch('os.path.exists')
  @mock.patch('slapos.runner.utils.updateInstanceParameter')
  @mock.patch('slapos.runner.utils.isSoftwareReleaseCompleted')
  @mock.patch('slapos.runner.utils.runSlapgridUntilSuccess')
  @mock.patch('slapos.runner.utils.isSoftwareRunning')
  @mock.patch('slapos.runner.utils.runSoftwareWithLock')
  def test_isSoftwareReleaseReady(self,
                                  mock_runSoftwareWithLock,
                                  mock_isSoftwareRunning,
                                  mock_runSlapgridUntilSuccess,
                                  mock_isSoftwareReleaseCompleted,
                                  mock_updateInstanceParameter,
                                  mock_path_exists):
    cwd = os.getcwd()
    config = {
      'etc_dir': cwd,
      'runner_workdir': cwd,
      'slapos-software': 'slapos/dummy',
      'auto_deploy': False,
      'autorun': False,
    }
    # Every parameter is False, so do nothing
    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '0')

    # We define a software, so from now we expect to build
    mock_path_exists.return_value = True

    # auto_deploy is True, so Software Release should build
    config.update({
      'auto_deploy': True,
    })

    # slapgrid-sr is running
    mock_isSoftwareRunning.return_value = True
    mock_isSoftwareReleaseCompleted.return_value = False
    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '2')
    
    # SR is not built, and slapgrid-sr is not running, so it should be started
    mock_isSoftwareRunning.return_value = False
    mock_isSoftwareReleaseCompleted.return_value = False

    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '2')
    self.assertTrue(mock_runSoftwareWithLock.called)

    mock_runSoftwareWithLock.reset_mock()

    # SR is built
    mock_isSoftwareReleaseCompleted.return_value = True
    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '1')

    # If autorun is True, Instance is expected to build too
    config.update({
      'autorun': True,
    })
    mock_isSoftwareRunning.return_value = False
    mock_isSoftwareReleaseCompleted.return_value = True

    self.assertEqual(runner_utils.isSoftwareReleaseReady(config), '1')
    mock_runSlapgridUntilSuccess.assert_called_with(config, 'instance')


  @unittest.skip('No scenario defined')
  def test_autoDeployWontEraseExistingInstances(self):
    raise NotImplementedError

  @unittest.skip('No scenario defined')
  def test_requestingInstanceCorrectlyPassesTypeAndParameters(self):
    raise NotImplementedError

  @unittest.skip('No scenario defined')
  def test_parametersAreCorrectlyUpdatedAndGivenToTheInstance(self):
    raise NotImplementedError


if __name__ == '__main__':
  random.seed()
  unittest.main()



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2017 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os.path
import tempfile
import shutil
from datetime import date

from slapos.securedelete import getAgumentParser, shred

class TestSecureDelete(unittest.TestCase):
  def setUp(self):
    _, self.remove_file = tempfile.mkstemp()
    _, self.remove_file2 = tempfile.mkstemp()
    self.link_name = os.path.join(os.path.dirname(self.remove_file),
                                  '%s_link' % os.path.basename(self.remove_file))
    with open(self.remove_file, 'w') as f:
      f.write("Skjsds@ßdjierhjzlalaa...")
    with open(self.remove_file2, 'w') as f:
      f.write("Skjsds@ßdjiesdsdrhjzlalaa...")

  def tearDown(self):
    if os.path.exists(self.remove_file):
      os.remove(self.remove_file)
    if os.path.exists(self.remove_file2):
      os.remove(self.remove_file2)
    if os.path.exists(self.link_name):
      os.unlink(self.link_name)

  def test_secure_remove_file(self):
    options = getAgumentParser().parse_args(['-n', '2', '-u', '-z', '--file', self.remove_file])
    passes = 2 + 1 # Option -z is used, plus one more pass
    result = shred(options)
    self.assertFalse(os.path.exists(self.remove_file))
    self.assertTrue("pass %s/%s" % (passes, passes) in result)
    self.assertTrue("%s: removed" % os.path.basename(self.remove_file) in result)

  def test_secure_remove_file_keep_file(self):
    options = getAgumentParser().parse_args(['-n', '2', '-z', '--file', self.remove_file])
    passes = 2 + 1 # Option -z is used, plus one more pass
    result = shred(options)
    self.assertTrue(os.path.exists(self.remove_file))
    self.assertTrue("pass %s/%s" % (passes, passes) in result)
    self.assertFalse("%s: removed" % os.path.basename(self.remove_file) in result)

  def test_secure_remove_file_non_zero(self):
    options = getAgumentParser().parse_args(['-n', '2', '-u', '--file', self.remove_file])
    passes = 2
    result = shred(options)
    self.assertFalse(os.path.exists(self.remove_file))
    self.assertTrue("pass %s/%s" % (passes, passes) in result)
    self.assertTrue("%s: removed" % os.path.basename(self.remove_file) in result)

  def test_secure_remove_file_check_exist(self):
    options = getAgumentParser().parse_args(['-n', '2', '-u', '-s', '--file', 'random.txt', self.remove_file])
    passes = 2
    result = shred(options)
    self.assertFalse(os.path.exists(self.remove_file))
    self.assertTrue("pass %s/%s" % (passes, passes) in result)
    self.assertTrue("%s: removed" % os.path.basename(self.remove_file) in result)

  def test_secure_remove_file_check_exist_false(self):
    options = getAgumentParser().parse_args(['-n', '2', '-u', '--file', 'random.txt'])
    passes = 2
    with self.assertRaises(RuntimeError):
      result = shred(options)

  def test_secure_remove_file_with_link(self):
    options = getAgumentParser().parse_args(['-n', '2', '-u', '--file', self.link_name])
    passes = 2
    os.symlink(self.remove_file, self.link_name)
    result = shred(options)
    # shred removed link and target file
    self.assertFalse(os.path.exists(self.remove_file))
    self.assertFalse(os.path.exists(self.link_name))
    self.assertTrue("pass %s/%s" % (passes, passes) in result)
    self.assertTrue("%s: removed" % os.path.basename(self.remove_file) in result)

  def test_secure_remove_file_multiple_files(self):
    options = getAgumentParser().parse_args(['-n', '2', '-u', '-z', '--file', self.remove_file, self.remove_file2])
    passes = 2 + 1 # Option -z is used, plus one more pass
    result = shred(options)
    self.assertFalse(os.path.exists(self.remove_file))
    self.assertTrue("pass %s/%s" % (passes, passes) in result)
    self.assertTrue("%s: removed" % os.path.basename(self.remove_file) in result)

    self.assertFalse(os.path.exists(self.remove_file2))
    self.assertTrue("%s: removed" % os.path.basename(self.remove_file2) in result)

if __name__ == '__main__':
  unittest.main()



# Command line script to test a RSS feed in a promise
# Checks that a given pattern can be found (or not) in the title or the
# description of the latest feed item.
# A time buffer option can be given, to determine if the emitter process is in
# a stalled state, in the case that no OK pattern has been found

import argparse
import datetime
import feedparser
import sys


def parseArguments():
  parser = argparse.ArgumentParser()
  parser.add_argument('--feed-path', dest='feed_path',
                      help='Path or Url of the feed to search')
  parser.add_argument('--title', dest='title', action='store_true',
                      help='Patterns should be looked for in feed item\'s title')
  parser.add_argument('--description', dest='description', action='store_true',
                      help='Patterns should be looked for in feed item\'s description')
  parser.add_argument('--ok-pattern', dest='ok_pattern_list', action='append',
                      default=[],
                      help='If this pattern is found, then promise succeeds')
  parser.add_argument('--ko-pattern', dest='ko_pattern_list', action='append',
                      default=[],
                      help='If this pattern is found, then promise fails')
  parser.add_argument('--time-buffer', dest='time_buffer', type=int,
                      default=0,
                      help='Time delta in seconds before the promise really succeeds or fails')
  return parser.parse_args()


def containsPattern(string, pattern_list):
  for pattern in pattern_list:
    if string.find(pattern) >= 0:
      return True
  return False


def checkFeedAsPromise(feed, option):
  feed = feedparser.parse(feed)

  if feed.bozo:
    return 'Feed malformed : %s (at line %s)' % (
      feed.bozo_exception.getMessage(),
      feed.bozo_exception.getLineNumber(),
    )

  if len(feed.entries) == 0:
    return ''

  last_item = feed.entries[-1]
  if option.title:
    candidate_string = last_item.title
  elif option.description:
    candidate_string = last_item.description
  else:
    return 'At least one in [--title|--description] should be provided'

  publication_date = datetime.datetime(*last_item.published_parsed[:7])
  publication_age = datetime.datetime.now() - publication_date
  time_buffer = datetime.timedelta(seconds=option.time_buffer)

  ok_pattern_found = containsPattern(candidate_string, option.ok_pattern_list)
  ko_pattern_found = containsPattern(candidate_string, option.ko_pattern_list)

  if ok_pattern_found and ko_pattern_found:
    return 'Both OK and KO patterns found: please check arguments'

  # Expectations fulfilled
  if ok_pattern_found:
      return ''

  if ko_pattern_found:
    return ("At least one of the failure patterns (%s) "
            "has been found in the last status (%s)" % (option.ko_pattern_list, candidate_string))

  if not ok_pattern_found:
    if publication_age < time_buffer:
      # We have to wait for buffer to expire
      return ''
    else:
      # If time-buffer is out, we are in stalled state
      return 'Stalled situation : Last update (%s) happened on %s' % (candidate_string, publication_date)

  # If not ok, and not stalled, what can have possibly happen ?
  return 'Something went wrong, check promise code'


def main():
  option = parseArguments()
  result = checkFeedAsPromise(option.feed_path, option)

  if len(result) > 0:
    sys.exit(result)
  else:
    sys.exit(0)


if __name__ == '__main__':
  main()



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2010, 2015 Nexedi SA and Contributors. All Rights Reserved.
#                    Nicolas Delaby <nicolas@nexedi.com>
#                    Romain Courteaud <romain@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import os
import shutil
import sys
import re
import shlex
from subprocess import Popen, PIPE, STDOUT
import logging
from datetime import datetime

_MARKER = []
WGET = 'wget'
WGET_ARGS = '-r -nc --retry-connrefused --save-headers'
# WGET_ARGS = '-r -nc --retry-connrefused --save-headers --header "Host: example.org" --no-check-certificate'
# WGET_FILL_CACHE_ARGS = '-q -O /dev/null  --header="Accept-Encoding: gzip, deflate"'

class ProcessTimeoutException(Exception):
  pass

def alarm_handler(signum, frame):
  raise ProcessTimeoutException

class HTTPCacheCheckerTestSuite(object):

  URL_CODE = 'url'
  STATUS_CODE = 'status'
  FILE_PATH_CODE = 'path'
  ERROR_CODE = 'error'
  OTHER_CODE = 'other'

  LOG_LEVEL_DICT = {'debug': logging.DEBUG,
                    'info': logging.INFO,
                    'warning': logging.WARNING,
                    'error': logging.ERROR,
                    'critical': logging.CRITICAL}

  REGEXP_URL_SEARCH = re.compile('^--\d{4}.*--\s+(?P<%s>.*)$' % URL_CODE)
  REGEXP_STATUS_SEARCH = re.compile(
    '^HTTP request sent, awaiting response\.\.\. (?P<%s>\d+).+$' % STATUS_CODE
  )
  REGEXP_SAVE_FILE = re.compile("^Saving to: '(?P<%s>.*)'" % FILE_PATH_CODE)
  REGEXP_ERROR = re.compile("^(ERROR|wget): (?P<%s>.*)" % ERROR_CODE)

  REGEXP_HEADER_CACHE = re.compile("Via: .*ApacheTrafficServer/.*\[(?P<hit>c.s.f.)\]", re.MULTILINE)
  generic_header_search_regex = '%s:\s(.*)\s$'

  ACCEPTABLE_STATUS_LIST = ('200', '304', '301', '302')

  def __init__(self, root_url, working_directory,
               header_list,
               debug_level, file_log_path, conditional_header_dict,
               no_header_dict, traffic_server_cache_list):
    """
      root_url : website to check
      working_directory : where fetched data will be downloaded
      header_list : Key == Header id.
                    value: if equals True means header
                             needs to be present in RESPONSE
                           if this is tuple, the Header value must sastify
                             at least one of the proposed values
      debug_level : log level of this utility (debug =>very verbose,
                                               info=>normal,
                                               warning=> nothing)
      file_log_path: path to log file
      conditional_header_dict : Key  == Section id (like 'header url='.*/login')
                                value: the configuration lines in this sction
                                       (config format is same as header_list)
      no_header_dict : Key == Section id (like 'no_header url=.*/sitemap')
                       value: = not exsiting headers
      traffic_server_cache_list: list of accepted traffic server cache result
    """
    self.root_url = root_url
    self.working_directory = working_directory
    self.header_list = header_list
    self.conditional_header_dict = conditional_header_dict
    self.no_header_dict = no_header_dict
    self.traffic_server_cache_list = traffic_server_cache_list
    level = self.LOG_LEVEL_DICT.get(debug_level, logging.INFO)
    logging.basicConfig(filename=file_log_path, level=level)
    self.report_dict = {}

  def _initFolder(self):
    """Delete and create workgin directory
    """
    if os.path.isdir(self.working_directory):
      logging.debug('Re-creating folder:%r' % self.working_directory)
      shutil.rmtree(self.working_directory)
    else:
      logging.debug('Creating folder:%r' % self.working_directory)
    os.mkdir(self.working_directory)

  def _runSpider(self):
    """Run wget in working_directory with headers in result
    """
    wget_command_string = '%s %s %s' % (WGET, WGET_ARGS, self.root_url)
    logging.debug('wget command:%r' % wget_command_string)
    wget_argument_list = shlex.split(wget_command_string)
    wget_process = Popen(wget_argument_list, stdin=PIPE,
                         stdout=PIPE, stderr=STDOUT,
                         env={'LANG': 'en_EN'}, # Force output messages
                                                # in English
                         universal_newlines=True,
                         cwd=self.working_directory) # working directory
    stdout, stderr = wget_process.communicate()
    return stdout

#   def _fillCache(self):
#     """Run wget to fill frontend cache
#     """
#     wget_command_string = '%s %s %s' % (WGET, WGET_ARGS, self.root_url)
#     logging.debug('wget command:%r' % wget_command_string)
#     wget_argument_list = shlex.split(wget_command_string)
#     wget_process = Popen(wget_argument_list, stdin=PIPE,
#                          stdout=PIPE, stderr=STDOUT,
#                          env={'LANG': 'en_EN'}, # Force output messages
#                                                 # in English
#                          universal_newlines=True,
#                          cwd=self.working_directory) # working directory
#     stdout, stderr = wget_process.communicate()
#     return stdout

  def _getHeaderPolicyList(self, url, fetched_data):
    """
    create the header checking policy list by the url, and the header

    [header_list]
    Last-Modified = True
    Vary = Accept-Language, Cookie, Accept-Encoding
    Cache-Control = max-age=300
    max-age=3600
    [header url=.*/contact_form]
    Last-Modified = True

    [no-header content-type=(image/.*|.*/javascript)]
    Vary = False

    [erp5_extension_list]
    prohibited_folder_name_list = web_page_module
    document_module
    prohibited_file_name_list = WebSection_viewAsWeb
    Base_viewHistory
    list
    """
    def getNoCheckHeaderList(url, fetched_data):
      """
        create no check header list
      """
      pick_content_type = re.compile('^no_header\s*content-type=(.*)')
      pick_url = re.compile('^no_header\s*url=(.*)')
      section_list = self.no_header_dict.keys()
      no_check_header_list = []
      for section in section_list:
        content_type_regex_str_match = pick_content_type.match(section)
        url_regex_str_match = pick_url.match(section)
        if content_type_regex_str_match is not None:
          content_type_regex_str = content_type_regex_str_match.group(1)
          content_type_regex = \
            re.compile('Content-Type:\s%s' % content_type_regex_str,
                       re.MULTILINE | re.IGNORECASE)
          if content_type_regex.search(fetched_data) is not None:
            for header, value in self.no_header_dict[section]:
              no_check_header_list.append(header)
          continue
        if url_regex_str_match is not None:
          url_regex_str = url_regex_str_match.group(1)
          if re.compile(url_regex_str).match(url) is not None:
            for header, value in self.no_header_dict[section]:
              no_check_header_list.append(header)
      return no_check_header_list

    def getConditionalHeaderDict(url, fetched_data):
      """ create header policy by the url and header"""
      conditional_header_dict = {}
      section_list = self.conditional_header_dict.keys()
      pick_content_type = re.compile('header\s*content-type=(.*)')
      pick_url = re.compile('header\s*url=(.*)')
      for section in section_list:
        content_type_regex_str_match = pick_content_type.match(section)
        url_regex_str_match = pick_url.match(section)
        if content_type_regex_str_match is not None:
          content_type_regex_str = content_type_regex_str_match.group(1)
          content_type_regex = \
            re.compile('Content-Type:\s%s' % content_type_regex_str,
                       re.MULTILINE | re.IGNORECASE)
          if content_type_regex.search(fetched_data) is not None:
            for header, value in self.conditional_header_dict[section]:
              conditional_header_dict[header] = _formatConfiguration(value)
          continue
        if url_regex_str_match is not None:
          url_regex_str = url_regex_str_match.group(1)
          if re.compile(url_regex_str).match(url) is not None:
            for header, value in self.conditional_header_dict[section]:
              conditional_header_dict[header] = _formatConfiguration(value)
      return conditional_header_dict

    validator_dict = {}
    no_header_list = getNoCheckHeaderList(url, fetched_data)
    conditional_header_dict = getConditionalHeaderDict(url, fetched_data)
    conditional_header_list = conditional_header_dict.keys()
    global_header_list = self.header_list.keys()
    header_policy_list = []
    if conditional_header_list:
      conditional_check_header_set = (set(conditional_header_list)
                                      - set(no_header_list))
      for header in conditional_check_header_set:
        header_policy_list.append((header, conditional_header_dict[header]))
    else:
      global_check_header_set = (set(global_header_list)
                                 - set(no_header_list))
      for header in global_check_header_set:
        header_policy_list.append((header, self.header_list[header]))
    # return items
    return header_policy_list

  def _validateHeader(self, url, header, reference_value, fetched_data):
    """validate header with the header policy"""
    re_compiled = re.compile(self.generic_header_search_regex % header,
                             re.MULTILINE | re.IGNORECASE)
    match_object = re_compiled.search(fetched_data)
    if match_object is None:
      message = 'header:%r not found' % (header)
      self.report_dict.setdefault(url, []).append(message)
    else:
      read_value = match_object.group(1)
      if reference_value is True and not read_value:
        message = 'value of header:%r not found' % (header)
        self.report_dict.setdefault(url, []).append(message)
      elif isinstance(reference_value, (tuple,list)):
        if read_value not in reference_value:
          message = 'value of header:%r does not match'\
                    ' (%r not in %r)' %\
                                    (header, read_value, reference_value)
          self.report_dict.setdefault(url, []).append(message)

  def _isSameUrl(self, url):
    """
      Return whether the url is already checked or not.

      Example case):
      http://example.com/login_form
      http://example.com/login_form/
    """
    if url in (None, ''):
      return False
    same_url = None
    if url.endswith('/'):
      same_url = url.rstrip('/')
    else:
      same_url = '%s/' % url
    if same_url in self.report_dict:
      return True
    return False

  def _parseWgetLine(self, line):
    """return tuple (code, value)
    code: 'url' means this is requested url
          'status' means we can read status code
          'other' something we can discard
    """
    for regexp, code in [
      (self.REGEXP_URL_SEARCH, self.URL_CODE),
      (self.REGEXP_STATUS_SEARCH, self.STATUS_CODE),
      (self.REGEXP_SAVE_FILE, self.FILE_PATH_CODE),
      (self.REGEXP_ERROR, self.ERROR_CODE)
      ]:
      match_object = regexp.search(line)
      if match_object is not None:
        return code, match_object.group(code)
    return self.OTHER_CODE, line

  def _parseWgetLogs(self, wget_log_file, discarded_url_list=_MARKER,
                     prohibited_file_name_list=None,
                     prohibited_folder_name_list=None):
    """read wget logs and test Caching configuration
    """
    if discarded_url_list is _MARKER:
      first_pass = True
    else:
      first_pass = False
    x_varnish_reference_list = []

    discarded = True
    for line in wget_log_file.splitlines():
      logging.debug('wget output:%r' % line)
      code, value = self._parseWgetLine(line)
      if code == self.URL_CODE:
        # This is the first Line by URL checked in wget stdout
        url = value
        logging.debug('url:%r' % url)
        discarded = False
        if not first_pass and url in discarded_url_list:
          # URL already checked during first pass
          logging.debug('%r Discarded' % url)
          discarded = True
        elif self._isSameUrl(url):
          discarded = True

      if discarded:
        # keep reading wget process without doing anything
        continue

      if code == self.ERROR_CODE:
        message = 'Page in error:%r' % (value)
        self.report_dict.setdefault(url, []).append(message)

      if code == self.STATUS_CODE:
        if value not in self.ACCEPTABLE_STATUS_LIST:
          message = 'Page in error status:%r' % (value)
          self.report_dict.setdefault(url, []).append(message)

      if code == self.FILE_PATH_CODE:
        logging.debug('checking:%r' % value)
        # Here we check if Response was cached
        file_path = os.path.sep.join((self.working_directory, value))
        folder_path , filename = os.path.split(file_path)
        if prohibited_file_name_list:
          if '?' in filename:
            filename = filename.rpartition('?')[0]
          if filename in prohibited_file_name_list:
            # Error
            message = '%r is prohibited as filename' % filename
            self.report_dict.setdefault(url, []).append(message)
        if prohibited_folder_name_list:
          for prohibited_folder_name in prohibited_folder_name_list:
            if prohibited_folder_name in folder_path.split(os.path.sep):
              message = '%r is prohibited as folder name' % prohibited_folder_name
              self.report_dict.setdefault(url, []).append(message)
              break
        file_object = None
        try:
          file_object = open(file_path, 'r')
        except IOError:
          # This is probably a folder try with /index.html
          index_file_path = file_path + os.path.sep + 'index.html'
          try:
            file_object = open(index_file_path, 'r')
          except IOError:
           # sometimes this is appended with .1
           number_file_path = file_path + '.1'
           try:
             file_object = open(number_file_path, 'r')
           except IOError:
             logging.info('File not found for url:%r %r' %\
                        (url, (file_path, index_file_path, number_file_path),))
             continue
        fetched_data = file_object.read()
        file_object.close()

        x_cache_header_match_object = \
          self.REGEXP_HEADER_CACHE.search(fetched_data)
        if x_cache_header_match_object is None:
          # This RESPONSE is not cached by Varnish
          message = 'Via cache header not found'
          self.report_dict.setdefault(url, []).append(message)
        else:
          # http://labs.omniti.com/people/mark/ats_sa/slides.html#slide-18
          cached = False
          for traffic_server_cache in self.traffic_server_cache_list:
            if (x_cache_header_match_object.group('hit').startswith(traffic_server_cache)):
              cached = True
              break

          if cached:
            # This is a cached content with a positive hit value
            # dot not check this url in second pass
            logging.debug('will be discarded:%r' % url)
            discarded_url_list.append(url)
          else:
            message = 'Not a cache hit! (got %s) (expect %s)' % (
              x_cache_header_match_object.group('hit'),
              self.traffic_server_cache_list
            )
            self.report_dict.setdefault(url, []).append(message)

        # parse the web checker configuration file and run the header
        # validation method
        for header, reference_value in self._getHeaderPolicyList(
                                              url, fetched_data):
          self._validateHeader(url, header, reference_value, fetched_data)

    return x_varnish_reference_list, discarded_url_list[:]

  def start(self, prohibited_file_name_list=None,
            prohibited_folder_name_list=None):
    """Run test suite
    return errors
    """
    logging.info('#' * 52)
    logging.info('date:%r' % datetime.now().isoformat())
    logging.info('#' * 52)

    self._initFolder()
    logging.info('Starting pass:%r' % self.root_url)
    wget_log_file = self._runSpider()
    x_varnish_reference_list, discarded_url_list =\
                           self._parseWgetLogs(wget_log_file,
                       prohibited_file_name_list=prohibited_file_name_list,
                       prohibited_folder_name_list=prohibited_folder_name_list)
    logging.info('End of second pass\n')

    if self.report_dict:
      report_message_list = ['*Errors*:']
      for url, message_list in self.report_dict.iteritems():
        unique_message_list = []
        for message in message_list:
          if message not in unique_message_list:
            unique_message_list.append(message)
        report_message_list.append('\n')
        report_message_list.append(url)
        report_message_list.extend(['\t%s' % message for message\
                                                       in unique_message_list])
      report_message = '\n'.join(report_message_list)
      signal = 'PROBLEM'
    else:
      report_message = 'No errors'
      signal = 'OK'
    subject = '%r:HTTP Cache checker results for %s' % (signal, self.root_url)
    return subject + '\n' + report_message


from optparse import OptionParser
import ConfigParser

def _formatConfiguration(configuration):
  """ format the configuration"""
  if configuration in ('True', 'true', 'yes'):
    return True
  return configuration.splitlines()

def web_checker_utility():
  usage = "usage: %prog [options] config_path"
  parser = OptionParser(usage=usage)
  parser.add_option('-o', '--output_file',
                    dest='output_file')

  (options, args) = parser.parse_args()
  if len(args) != 1 :
    print parser.print_help()
    parser.error('incorrect number of arguments')
  config_path = args[0]

  config = ConfigParser.RawConfigParser()
  config.read(config_path)

  working_directory = config.get('web_checker', 'working_directory')
  url = config.get('web_checker', 'url')
  debug_level = config.get('web_checker', 'debug_level')

  file_log_path = 'web_checker.log'
  if config.has_option('web_checker', 'file_log_path'):
    file_log_path = config.get('web_checker', 'file_log_path')

  if config.has_option('web_checker', 'traffic_server_cache'):
    traffic_server_cache_list = config.get('web_checker', 'traffic_server_cache').splitlines()
  else:
    traffic_server_cache_list = ['cHs']

  header_list = {}
  for header, configuration in config.items('header_list'):
    if header in config.defaults().keys():
      # defaults are shared for all sections.
      # so discard them from header_list
      continue
    value = _formatConfiguration(configuration)
    header_list[header] = value
  conditional_header_dict = {}
  no_header_dict = {}

  for section in config.sections():
    item_list = config.items(section)
    if re.compile("^header\s.*").match(section) is not None:
      conditional_header_dict.setdefault(section, []).extend(item_list)
    if re.compile("^no_header\s.*").match(section) is not None:
      no_header_dict.setdefault(section, []).extend(item_list)

  if config.has_section('erp5_extension_list'):
    prohibited_file_name_list = config.get('erp5_extension_list',
                                      'prohibited_file_name_list').splitlines()
    prohibited_folder_name_list = config.get('erp5_extension_list',
                                    'prohibited_folder_name_list').splitlines()
  else:
    prohibited_file_name_list = prohibited_folder_name_list = []

  instance = HTTPCacheCheckerTestSuite(url,
                                       working_directory,
                                       header_list,
                                       debug_level,
                                       file_log_path,
                                       conditional_header_dict,
                                       no_header_dict,
                                       traffic_server_cache_list)

  result = instance.start(prohibited_file_name_list=prohibited_file_name_list,
                       prohibited_folder_name_list=prohibited_folder_name_list)
  if options.output_file:
    file_object = open(options.output_file, 'w')
    file_object.write(result)
    file_object.close()
  else:
    print result

if __name__ == '__main__':
  sys.exit(web_checker_utility())
